import { ComboDisplayModel, ComboModel } from 'types';
import { Group, Tuple3Number } from '@antv/g';
import { DisplayMapper, LodStrategyObj, State } from '../types/item';
import { ComboStyleSet } from '../types/theme';
import { ComboModelData, ComboUserModelData } from '../types/combo';
import { Point } from '../types/common';
import Node from './node';
interface IProps {
    model: ComboModel;
    renderExtensions: any;
    containerGroup: Group;
    mapper?: DisplayMapper;
    stateMapper?: {
        [stateName: string]: DisplayMapper;
    };
    zoom?: number;
    theme: {
        styles: ComboStyleSet;
        lodStrategy: LodStrategyObj;
    };
    device?: any;
    onframe?: Function;
    onfinish?: Function;
    getCombinedBounds?: () => {
        center: Tuple3Number;
        min: Tuple3Number;
        max: Tuple3Number;
        size: Tuple3Number;
    } | false;
    getChildren?: () => (Node | Combo)[];
}
export default class Combo extends Node {
    type: 'combo';
    private anchorPointsCache;
    private cacheCombinedBounds?;
    getCombinedBounds: () => {
        center: Tuple3Number;
        min: Tuple3Number;
        max: Tuple3Number;
        size: Tuple3Number;
    } | false;
    getChildren: () => (Combo | Node)[];
    constructor(props: IProps);
    init(props: any): void;
    draw(displayModel: ComboDisplayModel, diffData?: {
        previous: ComboUserModelData;
        current: ComboUserModelData;
    }, diffState?: {
        previous: State[];
        current: State[];
    }, animate?: boolean, onfinish?: Function): void;
    /**
     * Sometimes no changes on edge data, but need to re-draw it
     * e.g. source and target nodes' position changed
     */
    forceUpdate: Function;
    /**
     * Maps (mapper will be function, value, or encode format) model to displayModel and find out the shapes to be update for incremental updating.
     * @param model inner model
     * @param diffData changes from graphCore changed event
     * @param isReplace whether replace the whole data or partial update
     * @returns
     */
    getDisplayModelAndChanges(innerModel: ComboModel, diffData?: {
        previous: ComboModelData;
        current: ComboModelData;
    }, isReplace?: boolean): {
        model: ComboDisplayModel;
        typeChange?: boolean;
    };
    private updateModelByBounds;
    getPosition(): Point;
    getIntersectPoint(point: Point, innerPoint: Point, anchorPoints: number[][]): Point;
    clone(containerGroup: Group, onlyKeyShape?: boolean, disableAnimate?: boolean, getCombinedBounds?: () => {
        center: Tuple3Number;
        min: Tuple3Number;
        max: Tuple3Number;
        size: Tuple3Number;
    } | false, getChildren?: () => (Node | Combo)[]): Group | Combo;
}
export {};
