import { Group } from '@antv/g';
import { EdgeDisplayModel, EdgeModel, ID } from '../types';
import { EdgeModelData } from '../types/edge';
import { DisplayMapper, State, LodStrategyObj } from '../types/item';
import { EdgeStyleSet } from '../types/theme';
import Item from './item';
import Node from './node';
import Combo from './combo';
interface IProps {
    model: EdgeModel;
    renderExtensions: any;
    containerGroup: Group;
    mapper?: DisplayMapper;
    stateMapper?: {
        [stateName: string]: DisplayMapper;
    };
    sourceItem: Node | Combo;
    targetItem: Node | Combo;
    zoom?: number;
    theme: {
        styles: EdgeStyleSet;
        lodStrategy: LodStrategyObj;
    };
    onframe?: Function;
    nodeMap?: Map<ID, Node>;
}
export default class Edge extends Item {
    destroyed: boolean;
    model: EdgeModel;
    displayModel: EdgeDisplayModel;
    /** Set to different value in implements */
    type: 'edge';
    nodeMap: Map<ID, Node>;
    sourceItem: Node | Combo;
    targetItem: Node | Combo;
    constructor(props: IProps);
    draw(displayModel: EdgeDisplayModel, diffData?: {
        previous: EdgeModelData;
        current: EdgeModelData;
    }, diffState?: {
        previous: State[];
        current: State[];
    }, animate?: boolean, onfinish?: Function): void;
    /**
     * Sometimes no changes on edge data, but need to re-draw it
     * e.g. source and target nodes' position changed
     */
    forceUpdate: Function;
    /**
     * Update end item for item and re-draw the edge
     * @param type update source or target
     * @param endItem new item to ended
     */
    updateEnd(type: 'source' | 'target', endItem: Node): void;
    clone(containerGroup: Group, sourceItem: Node | Combo, targetItem: Node | Combo, onlyKeyShape?: boolean, disableAnimate?: boolean): Group | Edge;
}
export {};
