import { IHook } from '../types/hook';
/**
 * A hook class unified the definitions of tap, untap, and emit.
 * One hook corresponds to one lifecyle on a graph.
 */
export default class Hook<T> implements IHook<T> {
    name: string;
    /**
     * A series listeners for this hook which will be executed one by one
     */
    listeners: ((param: T) => void)[];
    constructor(cfg: any);
    /**
     * Tap a listener to the corresponding lifecycle of this hook.
     * @param listener
     */
    tap(listener: (param: T) => void): void;
    /**
     * Remove a listener from the corresponding lifecycle of this hook.
     * @param listener
     */
    unTap(listener: (param: T) => void): void;
    /**
     * Emit the corresponding lifecycle to call the listeners
     * @param param
     */
    emit(param: T): void;
    /**
     * Linearly async emit the corresponding lifecycle to call the listeners
     * @param param
     */
    emitLinearAsync(param: T): Promise<void>;
}
