var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * A hook class unified the definitions of tap, untap, and emit.
 * One hook corresponds to one lifecyle on a graph.
 */
export default class Hook {
    constructor(cfg) {
        /**
         * A series listeners for this hook which will be executed one by one
         */
        this.listeners = [];
        const { name } = cfg;
        this.name = name;
    }
    /**
     * Tap a listener to the corresponding lifecycle of this hook.
     * @param listener
     */
    tap(listener) {
        this.listeners.push(listener);
    }
    /**
     * Remove a listener from the corresponding lifecycle of this hook.
     * @param listener
     */
    unTap(listener) {
        const idx = this.listeners.indexOf(listener);
        if (idx > -1)
            this.listeners.splice(idx, 1);
    }
    /**
     * Emit the corresponding lifecycle to call the listeners
     * @param param
     */
    emit(param) {
        this.listeners.forEach((listener) => listener(param));
    }
    /**
     * Linearly async emit the corresponding lifecycle to call the listeners
     * @param param
     */
    emitLinearAsync(param) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const listener of this.listeners) {
                yield listener(param);
            }
            // return new Promise(async () => {
            //   let start = Promise.resolve();
            //   this.listeners.forEach((listener) => {
            //     start = start.then(
            //       async () =>
            //         new Promise(async (resolve, reject) => {
            //           try {
            //             await listener(param);
            //             resolve();
            //           } catch (e) {
            //             reject();
            //           }
            //         }),
            //     );
            //   });
            // });
        });
    }
}
//# sourceMappingURL=hooks.js.map