import { Behavior } from '../../types/behavior';
const ALLOWED_TRIGGERS = ['click', 'dblclick'];
const DEFAULT_OPTIONS = {
    trigger: 'click',
    eventName: '',
    disableAnimate: false,
    shouldBegin: () => true,
    shouldUpdate: () => true,
};
export class CollapseExpandTree extends Behavior {
    constructor(options) {
        super(Object.assign({}, DEFAULT_OPTIONS, options));
        this.timeout = undefined;
        this.getEvents = () => {
            return this.options.trigger === 'dblclick'
                ? {
                    'node:dblclick': this.onClick,
                }
                : {
                    'node:click': this.onClickBesideDblClick,
                };
        };
        // Validate options
        if (options.trigger && !ALLOWED_TRIGGERS.includes(options.trigger)) {
            console.warn(`G6: Invalid trigger option "${options.trigger}" for collapse-expand-tree behavior!`);
            this.options.trigger = DEFAULT_OPTIONS.trigger;
        }
    }
    onClickBesideDblClick(event) {
        if (this.timeout) {
            clearTimeout(this.timeout);
            this.timeout = undefined;
            return;
        }
        this.timeout = setTimeout(() => {
            this.timeout = undefined;
            this.onClick(event);
        }, 200);
    }
    onClick(event) {
        if (!this.options.shouldBegin(event))
            return;
        const { itemId, itemType } = event;
        const { disableAnimate } = this.options;
        const model = this.graph.getNodeData(itemId);
        if (!model) {
            console.warn(`Node with id ${itemId} is not exist`);
            return;
        }
        this.graph.frontItem(itemId);
        let action = 'expand';
        if (model.data.collapsed) {
            this.graph.expand(itemId, disableAnimate);
        }
        else {
            this.graph.collapse(itemId, disableAnimate);
            action = 'collapse';
        }
        // Emit an event.
        if (this.options.eventName) {
            this.graph.emit(this.options.eventName, {
                action,
                itemId,
                itemType,
            });
        }
    }
}
//# sourceMappingURL=collapse-expand-tree.js.map