import { utils } from '..';
import { BrushSelect } from './brush-select';
const ALLOWED_TRIGGERS = ['drag', 'shift', 'ctrl', 'alt', 'meta'];
const LASSO_SHAPE_ID = 'g6-lasso-select-brush-shape';
export class LassoSelect extends BrushSelect {
    constructor() {
        super(...arguments);
        this.selectedIds = {
            nodes: [],
            edges: [],
            combos: [],
        };
        this.points = [];
        this.onMouseDown = (event) => {
            this.points = [];
            this.removeBrush();
            super.onMouseDown(event);
            if (this.beginPoint)
                this.points.push(this.beginPoint);
        };
        this.onMouseMove = (event) => {
            if (this.mousedown) {
                const { canvas: point } = event;
                this.points.push({ x: point.x, y: point.y });
            }
            super.onMouseMove(event);
        };
        this.onMouseUp = (event) => {
            const { canvas: point } = event;
            this.points.push({ x: point.x, y: point.y });
            super.onMouseUp(event);
            this.removeBrush();
            this.points = [];
        };
        this.updateBrush = (event) => {
            const { graph } = this;
            return graph.drawTransient('path', LASSO_SHAPE_ID, {
                style: {
                    path: this.getLassoPath(),
                },
            });
        };
        this.removeBrush = () => {
            const { graph } = this;
            graph.drawTransient('path', LASSO_SHAPE_ID, { action: 'remove' });
        };
        this.getLassoPath = () => {
            const points = this.points;
            const path = [];
            if (points.length) {
                points.forEach((point, index) => {
                    if (index === 0) {
                        path.push(['M', point.x, point.y]);
                    }
                    else {
                        path.push(['L', point.x, point.y]);
                    }
                });
                path.push(['L', points[0].x, points[0].y]);
            }
            return path;
        };
    }
    getSelector() {
        return utils.lassoSelector;
    }
    getPoints() {
        return this.points;
    }
    createBrush() {
        const { graph, options } = this;
        const { brushStyle } = options;
        return graph.drawTransient('path', LASSO_SHAPE_ID, {
            style: brushStyle,
            capture: false,
        });
    }
}
//# sourceMappingURL=lasso-select.js.map