import { Behavior } from '../../types/behavior';
import { IG6GraphEvent } from '../../types/event';
import { Point } from '../../types/common';
export interface RotateCanvas3DOptions {
    /**
     * The way to tranlate the canvas. 'drag' (default) means dragged by mouse, 'directionKeys' means the up/down/left/right key on keyBoard.
     */
    trigger?: 'drag' | 'directionKeys';
    /**
     * The assistance secondary key to trigger the behavior.
     * If it is not assigned, triggered only by trigger.
     * You can also assigned it with a key on keyboard e.g. 'shift',
     * to make the behavior triggered only when the key is pressing and trigger is happening.
     */
    secondaryKey?: string;
    /**
     * To speed up rotating while pressing and rotate the canvas by direciton keys (trigger = 'directionKeys').
     */
    speedUpKey?: string;
    /**
     * Whether allow zooming canvas on a node/edge/combo.
     */
    triggerOnItems?: boolean;
    /**
     * The event name to trigger when drag end.
     */
    eventName?: string;
    /**
     * Whether allow the behavior happen on the current item.
     */
    shouldBegin?: (event: IG6GraphEvent) => boolean;
}
/**
 * Rotate the 3d canvas with the center of the graph.
 */
export declare class RotateCanvas3D extends Behavior {
    pointStartAt: Point;
    keydown: boolean;
    speedUpKeydown: boolean;
    constructor(options: Partial<RotateCanvas3DOptions>);
    getEvents: () => {
        pointerdown: (event: any) => void;
        pointermove: (event: any) => void;
        pointerup: (event: any) => void;
        keydown: (event: any) => void;
        keyup: (event: any) => void;
    } | {
        keydown: (event: any) => void;
        keyup: (event: any) => void;
        pointerdown?: undefined;
        pointermove?: undefined;
        pointerup?: undefined;
    };
    onPointerDown(event: any): void;
    onPointerMove(event: any): void;
    onPointerUp(event: any): void;
    onKeydown(event: any): void;
    onKeyup(event: any): void;
    private rotate;
}
