import { CameraType } from '@antv/g';
import { RotateCanvas3D } from './rotate-canvas-3d';
// TODO: truck canvas
const VALID_TRIGGERS = ['drag', 'directionKeys'];
const DEFAULT_OPTIONS = {
    trigger: 'directionKeys',
    secondaryKey: '',
    eventName: '',
    shouldBegin: () => true,
};
/**
 * Translate the 3d canvas along the plane parallel to the screen.
 */
export class TrackCanvas3D extends RotateCanvas3D {
    constructor(options) {
        const finalOptions = Object.assign({}, DEFAULT_OPTIONS, options);
        if (!VALID_TRIGGERS.includes(finalOptions.trigger)) {
            console.warn(`The trigger ${finalOptions.trigger} is not valid for track-canvas-3d behavior, "drag" will take effect instead. Only "${VALID_TRIGGERS.join('", "')}" are available.`);
            finalOptions.trigger = 'drag';
        }
        super(finalOptions);
    }
    setTrackingCamera() {
        const { graph } = this;
        const camera = graph.canvas.getCamera();
        this.previousType = camera.type;
        graph.canvas.getCamera().setType(CameraType.TRACKING);
    }
    restoreCameraType() {
        if (this.previousType !== undefined) {
            const { graph } = this;
            graph.canvas.getCamera().setType(this.previousType);
        }
    }
    onPointerDown(event) {
        this.setTrackingCamera();
        super.onPointerDown(event);
    }
    onPointerUp(event) {
        this.restoreCameraType();
        super.onPointerUp(event);
    }
    onKeydown(event) {
        const { trigger, secondaryKey } = this.options;
        if (trigger === 'directionKeys') {
            if (!secondaryKey || (secondaryKey && this.keydown)) {
                this.setTrackingCamera();
            }
        }
        super.onKeydown(event);
    }
    onKeyup(event) {
        const { trigger, secondaryKey } = this.options;
        if (trigger === 'directionKeys') {
            if (!secondaryKey || (secondaryKey && this.keydown)) {
                this.restoreCameraType();
            }
        }
        super.onKeyup(event);
    }
}
//# sourceMappingURL=track-canvas-3d.js.map