import { Behavior } from '../../types/behavior';
import { IG6GraphEvent } from '../../types/event';
export interface ZoomCanvas3DOptions {
    /**
     * The way to tranlate the canvas. 'drag' (default) means dragged by mouse, 'directionKeys' means the up/down/left/right key on keyBoard.
     */
    trigger?: 'wheel' | 'upDownKeys';
    /**
     * The assistance secondary key to trigger the behavior.
     * If it is not assigned, triggered only by trigger.
     * You can also assigned it with a key on keyboard e.g. 'shift',
     * to make the behavior triggered only when the key is pressing and trigger is happening.
     */
    secondaryKey?: string;
    /**
     * Sensitivity of zooming.
     */
    sensitivity?: number;
    /**
     * Whether allow zooming canvas on a node/edge/combo.
     */
    triggerOnItems?: boolean;
    /**
     * The event name to trigger when drag end.
     */
    eventName?: string;
    /**
     * The min value of camera's dolly to constrain the zoom-canvas-3d behavior
     */
    minZoom?: number;
    /**
     * The max value of camera's dolly to constrain the zoom-canvas-3d behavior
     */
    maxZoom?: number;
    /**
     * Whether allow the behavior happen on the current item.
     */
    shouldBegin?: (event: IG6GraphEvent) => boolean;
}
/**
 * Zoom the 3d canvas along the ray vertical to the screen.
 */
export declare class ZoomCanvas3D extends Behavior {
    private keydown;
    constructor(options: Partial<ZoomCanvas3DOptions>);
    getEvents: () => {
        wheel: (event: any) => void;
        keydown: (event: any) => void;
        keyup: (event: any) => void;
    } | {
        keydown: (event: any) => void;
        keyup: (event: any) => void;
        wheel?: undefined;
    };
    onWheel(event: any): void;
    onKeydown(event: any): void;
    onKeyup(event: any): void;
}
