import { IG6GraphEvent } from 'types';
import { Behavior } from '../../types/behavior';
export interface ZoomCanvasOptions {
    /**
     * Whether enable optimize strategies, which will hide all the shapes excluding node keyShape while zooming.
     * TODO: optimize when trigger is upDownKeys
     */
    enableOptimize?: boolean;
    /**
     * Whether allow trigger this behavior when wheeling start on nodes / edges / combos.
     */
    triggerOnItems?: boolean;
    /**
     * The trigger for the behavior, 'wheel' by default. 'upDownKeys' means trigger this behavior by up / down keys on keyboard.
     */
    trigger?: 'wheel' | 'upDownKeys';
    /**
     * The assistant secondary key on keyboard. If it is not assigned, the behavior will be triggered when trigger happens.
     */
    secondaryKey?: string;
    /**
     * The key on keyboard to speed up translating while pressing and zoom-canvas by direction keys. The trigger should be 'directionKeys' for this option.
     */
    speedUpKey?: string;
    /**
     * The sensitivity / speed of zooming.
     */
    sensitivity?: number;
    /**
     * The event name to trigger when zoom end.
     */
    eventName?: string;
    /**
     * The min value of zoom ratio to constrain the zoom-canvas-3d behavior
     */
    minZoom?: number;
    /**
     * The max value of zoom ratio to constrain the zoom-canvas-3d behavior
     */
    maxZoom?: number;
    /**
     * Whether allow the behavior happen on the current item.
     */
    shouldBegin?: (event: IG6GraphEvent) => boolean;
}
export declare class ZoomCanvas extends Behavior {
    private zooming;
    private keydown;
    private speedupKeydown;
    private hiddenEdgeIds;
    private hiddenNodeIds;
    private zoomTimer;
    constructor(options: Partial<ZoomCanvasOptions>);
    getEvents(): {
        keydown: (event: any) => void;
        keyup: (event: any) => void;
        wheel?: undefined;
    } | {
        wheel: (event: any) => void;
        keydown: (event: any) => void;
        keyup: (event: any) => void;
    };
    private hideShapes;
    private endZoom;
    onWheel(event: any): void;
    onKeydown(event: any): void;
    onKeyup(event: any): void;
}
