var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { isArray, isNumber, uniqueId } from '@antv/util';
/**
 * Validate and format the graph data.
 * @param data input user data.
 * @param userGraphCore the graph core stores the previous data.
 * @returns formatted data.
 */
export const TransformV4Data = (data, options = {}, userGraphCore) => {
    const { nodes = [], edges = [], combos = [] } = data;
    const formattedNodes = nodes.map((node) => {
        const { id, type, style, size, color, label, labelCfg = {}, icon, linkPoints } = node, others = __rest(node, ["id", "type", "style", "size", "color", "label", "labelCfg", "icon", "linkPoints"]);
        const anchorShapes = [];
        if (linkPoints) {
            const { top, bottom, left, right } = linkPoints, otherAnchorShapeStyles = __rest(linkPoints, ["top", "bottom", "left", "right"]);
            Object.keys({ top, bottom, left, right }).forEach((key) => {
                if (!linkPoints[key])
                    return;
                anchorShapes.push(Object.assign({ position: key }, otherAnchorShapeStyles));
            });
        }
        const { style: labelStyle = {} } = labelCfg, otherLabelStyle = __rest(labelCfg, ["style"]);
        const { background } = labelStyle;
        const newNode = {
            id,
            data: Object.assign(Object.assign({}, others), { labelShape: Object.assign(Object.assign({}, labelStyle), otherLabelStyle) }),
        };
        if (type) {
            newNode.data.type = type.includes('-node') ? type : `${type}-node`;
        }
        if (style) {
            newNode.data.keyShape = style;
        }
        const nodeSize = formatSize(size);
        if (nodeSize) {
            newNode.data.keyShape = Object.assign(Object.assign({}, newNode.data.keyShape), { r: nodeSize === null || nodeSize === void 0 ? void 0 : nodeSize[0], width: nodeSize === null || nodeSize === void 0 ? void 0 : nodeSize[0], height: nodeSize === null || nodeSize === void 0 ? void 0 : nodeSize[1] });
        }
        if (color) {
            newNode.data.keyShape.fill = color;
        }
        if (label) {
            newNode.data.labelShape.text = label;
        }
        if (icon === null || icon === void 0 ? void 0 : icon.show) {
            newNode.data.icon = icon;
        }
        if (anchorShapes.length) {
            newNode.data.anchorShapes = anchorShapes;
        }
        if (background) {
            newNode.data.labelBackgroundShape = background;
        }
        return newNode;
    });
    const formattedEdges = edges.map((edge) => {
        const { id = uniqueId(), type, source, target, style, size, color, label, labelCfg = {}, icon, linkPoints } = edge, others = __rest(edge, ["id", "type", "source", "target", "style", "size", "color", "label", "labelCfg", "icon", "linkPoints"]);
        const { style: labelStyle = {} } = labelCfg, otherLabelStyle = __rest(labelCfg, ["style"]);
        const { background } = labelStyle;
        const newEdge = {
            id,
            source,
            target,
            data: Object.assign(Object.assign({}, others), { labelShape: Object.assign(Object.assign({}, labelStyle), otherLabelStyle) }),
        };
        if (type) {
            newEdge.data.type = type.includes('-edge') ? type : `${type}-edge`;
        }
        if (style) {
            newEdge.data.keyShape = style;
        }
        if (size) {
            newEdge.data.keyShape = newEdge.data.keyShape || {};
            newEdge.data.keyShape.lineWidth = size;
        }
        if (color) {
            newEdge.data.keyShape = newEdge.data.keyShape || {};
            newEdge.data.keyShape.stroke = color;
        }
        if (label) {
            newEdge.data.labelShape.text = label;
        }
        if (background) {
            newEdge.data.labelBackgroundShape = background;
        }
        return newEdge;
    });
    const formattedCombos = combos.map((combo) => {
        const { id, type, style, size, color, label, labelCfg = {}, icon, linkPoints } = combo, others = __rest(combo, ["id", "type", "style", "size", "color", "label", "labelCfg", "icon", "linkPoints"]);
        const { style: labelStyle = {} } = labelCfg, otherLabelStyle = __rest(labelCfg, ["style"]);
        const { background } = labelStyle;
        const newCombo = {
            id,
            data: Object.assign(Object.assign({}, others), { labelShape: Object.assign(Object.assign({}, labelStyle), otherLabelStyle) }),
        };
        if (type) {
            newCombo.data.type = type.includes('-combo') ? type : `${type}-combo`;
        }
        if (style) {
            newCombo.data.keyShape = style;
        }
        const comboSize = formatSize(size);
        if (comboSize) {
            newCombo.data.keyShape = Object.assign(Object.assign({}, newCombo.data.keyShape), { r: comboSize === null || comboSize === void 0 ? void 0 : comboSize[0], width: comboSize === null || comboSize === void 0 ? void 0 : comboSize[0], height: comboSize === null || comboSize === void 0 ? void 0 : comboSize[1] });
        }
        if (color) {
            newCombo.data.keyShape = newCombo.data.keyShape || {};
            newCombo.data.keyShape.fill = color;
        }
        if (label) {
            newCombo.data.labelShape.text = label;
        }
        if (background) {
            newCombo.data.labelBackgroundShape = background;
        }
        return newCombo;
    });
    return {
        nodes: formattedNodes || [],
        edges: formattedEdges || [],
        combos: formattedCombos || [],
    };
};
const formatSize = (size) => {
    if (!size)
        return;
    if (isNumber(size))
        return [size, size];
    if (isArray(size))
        return size.length > 2 ? [size[0], size[1]] : size;
    return;
};
//# sourceMappingURL=transformV4Data.js.map