import { BaseNode } from '../node/base';
export class CircleCombo extends BaseNode {
    constructor(props) {
        super(props);
        this.defaultStyles = {
            keyShape: {
                r: 16,
                x: 0,
                y: 0,
            },
        };
        // suggest to merge default styles like this to avoid style value missing
        // this.defaultStyles = mergeStyles([this.baseDefaultStyles, this.defaultStyles]);
    }
    draw(model, shapeMap, diffData, diffState) {
        const { data = {} } = model;
        let shapes = { keyShape: undefined };
        // keyShape
        shapes.keyShape = this.drawKeyShape(model, shapeMap, diffData);
        // haloShape
        if (data.haloShape && this.drawHaloShape) {
            shapes.haloShape = this.drawHaloShape(model, shapeMap, diffData);
        }
        // labelShape
        if (data.labelShape) {
            shapes.labelShape = this.drawLabelShape(model, shapeMap, diffData);
        }
        // labelBackgroundShape
        if (data.labelBackgroundShape) {
            shapes.labelBackgroundShape = this.drawLabelBackgroundShape(model, shapeMap, diffData);
        }
        // anchor shapes
        if (data.anchorShapes) {
            const anchorShapes = this.drawAnchorShapes(model, shapeMap, diffData, diffState);
            shapes = Object.assign(Object.assign({}, shapes), anchorShapes);
        }
        // iconShape
        if (data.iconShape) {
            shapes.iconShape = this.drawIconShape(model, shapeMap, diffData);
        }
        // badgeShape
        if (data.badgeShapes) {
            const badgeShapes = this.drawBadgeShapes(model, shapeMap, diffData, diffState);
            shapes = Object.assign(Object.assign({}, shapes), badgeShapes);
        }
        // otherShapes
        if (data.otherShapes && this.drawOtherShapes) {
            shapes = Object.assign(Object.assign({}, shapes), this.drawOtherShapes(model, shapeMap, diffData));
        }
        return shapes;
    }
    drawKeyShape(model, shapeMap, diffData, diffState) {
        return this.upsertShape('circle', 'keyShape', this.mergedStyles.keyShape, shapeMap, model);
    }
}
//# sourceMappingURL=circle.js.map