import { CubicEdge } from './cubic';
export class CubicVerticalEdge extends CubicEdge {
    constructor(props) {
        super(props);
        this.type = 'cubic-vertical-edge';
        this.defaultStyles = {
            keyShape: {
                x1: 0,
                y1: 0,
                z1: 0,
                x2: 0,
                y2: 0,
                z2: 0,
                isBillboard: true,
            },
        };
        /**
         * control point calculated according to startPoint, endPoint, percent, and offset
         * @param  {Point} startPoint source point position of edge (x, y)
         * @param  {Point} endPoint  target point position of edge (x, y)
         * @param  {Number} percent   the proportion of control points' in the segment, Range 0 to 1
         * @param  {Number} offset    the curveOffset
         * @param controlPoints the control point position
         * @returns control points
         */
        this.getControlPoints = (startPoint, endPoint, percent = 0.5, controlPoints, offset) => {
            if ((controlPoints === null || controlPoints === void 0 ? void 0 : controlPoints.length) > 1)
                return controlPoints;
            const distanceToPoint = (startPoint.y - endPoint.y) * percent;
            const controlPoint1 = this.getControlPoint(startPoint, endPoint, distanceToPoint);
            const controlPoint2 = this.getControlPoint(endPoint, startPoint, -distanceToPoint);
            return [controlPoint1, controlPoint2];
        };
        /**
         * control point calculated according to startPoint, endPoint, percent, and offset
         * @param  {Point} startPoint source point position of edge (x, y)
         * @param  {Point} endPoint  target point position of edge (x, y)
         * @param  {Number} percent   the proportion of control points' in the segment, Range 0 to 1
         * @param  {Number} offset    the curveOffset
         * @return {Point} control point (x,y)
         */
        this.getControlPoint = (startPoint, endPoint, offset = 0) => {
            if (endPoint.y === startPoint.y || endPoint.x === startPoint.x) {
                return { x: startPoint.x, y: (startPoint.y + endPoint.y) / 2 };
            }
            const point = {
                x: startPoint.x,
                y: startPoint.y - offset,
            };
            return point;
        };
        // suggest to merge default styles like this to avoid style value missing
        // this.defaultStyles = mergeStyles([this.baseDefaultStyles, this.defaultStyles]);
    }
}
//# sourceMappingURL=cubic-vertical.js.map