import { DisplayObject } from '@antv/g';
import { Point } from '../../../types/common';
import { EdgeModelData, EdgeShapeMap } from '../../../types/edge';
import { State } from '../../../types/item';
import { EdgeDisplayModel } from '../../../types';
import { BaseEdge } from './base';
export declare class CubicEdge extends BaseEdge {
    type: string;
    defaultStyles: {
        keyShape: {
            x1: number;
            y1: number;
            z1: number;
            x2: number;
            y2: number;
            z2: number;
            isBillboard: boolean;
        };
    };
    constructor(props: any);
    draw(model: EdgeDisplayModel, sourcePoint: Point, targetPoint: Point, shapeMap: EdgeShapeMap, diffData?: {
        previous: EdgeModelData;
        current: EdgeModelData;
    }, diffState?: {
        previous: State[];
        current: State[];
    }): {
        [otherShapeId: string]: DisplayObject<any, any>;
        keyShape: DisplayObject<any, any>;
        labelShape?: DisplayObject<any, any>;
        iconShape?: DisplayObject<any, any>;
    };
    drawKeyShape(model: EdgeDisplayModel, sourcePoint: Point, targetPoint: Point, shapeMap: EdgeShapeMap, diffData?: {
        previous: EdgeModelData;
        current: EdgeModelData;
    }, diffState?: {
        previous: State[];
        current: State[];
    }): DisplayObject<any, any>;
    /**
     * calculate the control points by curvePosition|controlPoints|curveOffset
     * @param startPoint: source point position of edge
     * @param endPoint target point position of edge
     * @param percent the proportion of control points' in the segment, Range 0 to 1
     * @param controlPoints the control point position
     * @param offset the curveOffset
     * @returns control points
     */
    protected getControlPoints: (startPoint: Point, endPoint: Point, percent: number, controlPoints: Point[], offset: number) => Point[];
    /**
     * control point calculated according to startPoint, endPoint, percent, and offset
     * @param  {Point} startPoint source point position of edge (x, y)
     * @param  {Point} endPoint  target point position of edge (x, y)
     * @param  {Number} percent   the proportion of control points' in the segment, Range 0 to 1
     * @param  {Number} offset    the curveOffset
     * @return {Point} control point (x,y)
     */
    protected getControlPoint: (startPoint: Point, endPoint: Point, percent: number, offset: number) => Point;
}
