var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { getControlPoint } from '../../../util/path';
import { BaseEdge } from './base';
export class QuadraticEdge extends BaseEdge {
    constructor(props) {
        super(props);
        this.type = 'quadratic-edge';
        this.defaultStyles = {
            keyShape: {
                controlPoints: [],
                curvePosition: 0.5,
                curveOffset: 30,
                isBillboard: true,
            },
        };
        /**
         * calculate the control point by curvePosition|controlPoints|curveOffset
         * @param startPoint: source point position of edge
         * @param endPoint target point position of edge
         * @param percent the proportion of control points' in the segment, Range 0 to 1
         * @param controlPoints the control point position
         * @param offset the curveOffset
         * @returns control points
         */
        this.getControlPoints = (startPoint, endPoint, percent = 0, controlPoints, offset) => {
            if (controlPoints === null || controlPoints === void 0 ? void 0 : controlPoints.length)
                return controlPoints;
            return [
                getControlPoint(startPoint, endPoint, percent, offset),
            ];
        };
        // suggest to merge default styles like this to avoid style value missing
        // this.defaultStyles = mergeStyles([this.baseDefaultStyles, this.defaultStyles]);
    }
    draw(model, sourcePoint, targetPoint, shapeMap, diffData, diffState) {
        const { data = {} } = model;
        const shapes = { keyShape: undefined };
        shapes.keyShape = this.drawKeyShape(model, sourcePoint, targetPoint, shapeMap, diffData);
        if (data.haloShape) {
            shapes.haloShape = this.drawHaloShape(model, shapeMap, diffData);
        }
        if (data.labelShape) {
            shapes.labelShape = this.drawLabelShape(model, shapeMap, diffData);
        }
        // labelBackgroundShape
        if (data.labelBackgroundShape) {
            shapes.labelBackgroundShape = this.drawLabelBackgroundShape(model, shapeMap, diffData);
        }
        if (data.iconShape) {
            shapes.iconShape = this.drawIconShape(model, shapeMap, diffData);
        }
        // TODO: other shapes
        return shapes;
    }
    drawKeyShape(model, sourcePoint, targetPoint, shapeMap, diffData, diffState) {
        const { keyShape: keyShapeStyle } = this.mergedStyles;
        const { startArrow, endArrow } = keyShapeStyle, others = __rest(keyShapeStyle, ["startArrow", "endArrow"]);
        const controlPoint = this.getControlPoints(sourcePoint, targetPoint, keyShapeStyle.curvePosition, keyShapeStyle.controlPoints, keyShapeStyle.curveOffset)[0];
        const lineStyle = Object.assign(Object.assign({}, others), { path: [
                ['M', sourcePoint.x, sourcePoint.y],
                ['Q', controlPoint.x, controlPoint.y, targetPoint.x, targetPoint.y],
            ] });
        this.upsertArrow('start', startArrow, others, model, lineStyle);
        this.upsertArrow('end', endArrow, others, model, lineStyle);
        return this.upsertShape('path', 'keyShape', lineStyle, shapeMap, model);
    }
}
//# sourceMappingURL=quadratic.js.map