var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { upsertShape3D } from '../../../util/shape3d';
import { BaseNode } from './base';
export class BaseNode3D extends BaseNode {
    constructor(props) {
        super(props);
        /**
         * The listener for graph zooming.
         * 1. show / hide some shapes while zoom level changed;
         * 2. change the shapes' sizes to make them have same visual size while zooming, e.g. labelShape, labelBackgroundShape.
         * @param shapeMap The shape map that contains all of the elements to show on the node.
         * @param zoom The zoom level of the graph.
         */
        this.onZoom = (shapeMap, zoom) => { };
        this.device = props.device;
    }
    /**
     * Draw the label shape of the 3D node
     * @param model The displayed model of this 3D node, only for drawing and not received by users.
     * @param shapeMap The shape map that contains all of the elements to show on the 3D node.
     * @param diffData An object that contains previous and current data.
     * @param diffState An object that contains previous and current 3D node's state.
     * @returns The display object representing the label shape of the 3D node.
     */
    drawLabelShape(model, shapeMap, diffData, diffState) {
        const { keyShape } = shapeMap;
        const { r, width, height, depth, x, y, z } = keyShape.attributes;
        const keyShapeBox = {
            center: [x, y, z],
            min: [x - r || width / 2, y - r || height / 2, z - r || depth / 2],
            max: [x + r || width / 2, y + r || height / 2, z + r || depth / 2],
        };
        const { labelShape: shapeStyle } = this.mergedStyles;
        const { position, offsetX: propsOffsetX, offsetY: propsOffsetY, maxWidth } = shapeStyle, otherStyle = __rest(shapeStyle, ["position", "offsetX", "offsetY", "maxWidth"]);
        const positionPreset = {
            x: keyShapeBox.center[0],
            y: keyShapeBox.max[1],
            z: keyShapeBox.center[2],
            textBaseline: 'top',
            textAlign: 'center',
            offsetX: 0,
            offsetY: 0,
            // wordWrapWidth,
        };
        switch (position) {
            case 'top':
                positionPreset.y = keyShapeBox.min[1];
                positionPreset.textBaseline = 'bottom';
                positionPreset.offsetY = -4;
                break;
            case 'left':
                positionPreset.x = keyShapeBox.min[0];
                positionPreset.y = keyShapeBox.center[1];
                positionPreset.textAlign = 'right';
                positionPreset.textBaseline = 'middle';
                positionPreset.offsetX = -8;
                break;
            case 'right':
                positionPreset.x = keyShapeBox.max[0];
                positionPreset.y = keyShapeBox.center[1];
                positionPreset.textAlign = 'left';
                positionPreset.textBaseline = 'middle';
                positionPreset.offsetX = 8;
                break;
            default: // at bottom by default
                positionPreset.offsetY = 4;
                break;
        }
        const offsetX = (propsOffsetX === undefined ? positionPreset.offsetX : propsOffsetX);
        const offsetY = (propsOffsetY === undefined ? positionPreset.offsetY : propsOffsetY);
        positionPreset.x += offsetX;
        positionPreset.y += offsetY;
        const style = Object.assign(Object.assign(Object.assign(Object.assign({}, this.defaultStyles.labelShape), positionPreset), { isBillboard: true }), otherStyle);
        return this.upsertShape('text', 'labelShape', style, shapeMap);
    }
    /**
     * Draw the icon shape of the 3D node
     * @param model The displayed model of this 3D node, only for drawing and not received by users.
     * @param shapeMap The shape map that contains all of the elements to show on the 3D node.
     * @param diffData An object that contains previous and current data.
     * @param diffState An object that contains previous and current 3D node's state.
     * @returns The display object representing the icon shape of the 3D node.
     */
    drawIconShape(model, shapeMap, diffData, diffState) {
        return super.drawIconShape(model, shapeMap, diffData, diffState);
    }
    // TODO: 3d billboard
    drawHaloShape(model, shapeMap, diffData, diffState) {
        const { keyShape } = shapeMap;
        const { haloShape: haloShapeStyle } = this.mergedStyles;
        if (haloShapeStyle.visible === false)
            return;
        const { nodeName, attributes } = keyShape;
        return this.upsertShape(nodeName, 'haloShape', Object.assign(Object.assign(Object.assign({}, attributes), { stroke: attributes.fill }), haloShapeStyle), shapeMap);
    }
    /**
     * 3D node does not support anchor shapes.
     * @param model
     * @param shapeMap
     * @param diffData
     * @param diffState
     * @returns
     */
    drawAnchorShapes(model, shapeMap, diffData, diffState) {
        return {};
    }
    drawBadgeShapes(model, shapeMap, diffData, diffState) {
        return super.drawBadgeShapes(model, shapeMap, diffData, diffState);
    }
    // TODO: 3d shapes?
    drawOtherShapes(model, shapeMap, diffData, diffState) {
        return {};
    }
    // TODO: 如何禁止重写？
    upsertShape(type, id, style, shapeMap) {
        return upsertShape3D(type, id, style, shapeMap, this.device);
    }
}
//# sourceMappingURL=base3d.js.map