import { BaseNode } from './base';
export class HexagonNode extends BaseNode {
    constructor(props) {
        super(props);
        this.defaultStyles = {
            keyShape: {
                r: 20,
                x: 0,
                y: 0,
                direction: 'horizontal',
            },
        };
        // suggest to merge default styles like this to avoid style value missing
        // this.defaultStyles = mergeStyles([this.baseDefaultStyles, this.defaultStyles]);
    }
    draw(model, shapeMap, diffData, diffState) {
        const { data = {} } = model;
        let shapes = { keyShape: undefined };
        // keyShape
        shapes.keyShape = this.drawKeyShape(model, shapeMap, diffData);
        // haloShape
        if (data.haloShape && this.drawHaloShape) {
            shapes.haloShape = this.drawHaloShape(model, shapeMap, diffData);
        }
        // labelShape
        if (data.labelShape) {
            shapes.labelShape = this.drawLabelShape(model, shapeMap, diffData);
        }
        // labelBackgroundShape
        if (data.labelBackgroundShape) {
            shapes.labelBackgroundShape = this.drawLabelBackgroundShape(model, shapeMap, diffData);
        }
        // anchor shapes
        if (data.anchorShapes) {
            const anchorShapes = this.drawAnchorShapes(model, shapeMap, diffData, diffState);
            shapes = Object.assign(Object.assign({}, shapes), anchorShapes);
        }
        // iconShape
        if (data.iconShape) {
            shapes.iconShape = this.drawIconShape(model, shapeMap, diffData);
        }
        // badgeShape
        if (data.badgeShapes) {
            const badgeShapes = this.drawBadgeShapes(model, shapeMap, diffData, diffState);
            shapes = Object.assign(Object.assign({}, shapes), badgeShapes);
        }
        // otherShapes
        if (data.otherShapes && this.drawOtherShapes) {
            shapes = Object.assign(Object.assign({}, shapes), this.drawOtherShapes(model, shapeMap, diffData));
        }
        return shapes;
    }
    drawKeyShape(model, shapeMap, diffData, diffState) {
        const { keyShape: keyShapeStyle } = this.mergedStyles;
        return this.upsertShape('polygon', 'keyShape', Object.assign(Object.assign({}, this.mergedStyles.keyShape), { points: this.getHexagonPoints(keyShapeStyle.r, keyShapeStyle.direction) }), shapeMap, model);
    }
    getHexagonPoints(r, direction) {
        const angleIncrement = Math.PI / 3; //The angle increment between vertex.
        const v = [];
        const offsetAngle = direction === 'horizontal' ? 0 : Math.PI / 2;
        for (let i = 0; i < 6; i++) {
            const angle = i * angleIncrement + offsetAngle;
            const vx = r * Math.cos(angle);
            const vy = r * Math.sin(angle);
            v.push([vx, vy]);
        }
        return v;
    }
    calculateAnchorPosition(keyShapeStyle) {
        const anchorPositionHorizontal = [
            'right',
            'rightbottom',
            'leftbottom',
            'left',
            'lefttop',
            'righttop',
        ];
        const anchorPositionVertical = [
            'bottom',
            'leftbottom',
            'lefttop',
            'top',
            'righttop',
            'rightbottom',
        ];
        const anchorPositionDirection = keyShapeStyle.direction === 'horizontal'
            ? anchorPositionHorizontal
            : anchorPositionVertical;
        const angleIncrement = Math.PI / 3; //The angle increment between vertex.
        const offsetAngle = keyShapeStyle.direction === 'horizontal' ? 0 : Math.PI / 2;
        const r = keyShapeStyle.r;
        const anchorPositionMap = {};
        for (let i = 0; i < 6; i++) {
            const angle = i * angleIncrement + offsetAngle;
            const vx = r * Math.cos(angle);
            const vy = r * Math.sin(angle);
            anchorPositionMap[anchorPositionDirection[i]] = [vx, vy];
        }
        return anchorPositionMap;
    }
}
//# sourceMappingURL=hexagon.js.map