import { convertToNumber } from '../../../util/type';
import { BaseNode } from './base';
export class RectNode extends BaseNode {
    constructor(props) {
        super(props);
        this.defaultStyles = {
            keyShape: {
                x: 0,
                y: 0,
                width: 32,
                height: 32,
            },
        };
        // suggest to merge default styles like this to avoid style value missing
        // this.defaultStyles = mergeStyles([this.baseDefaultStyles, this.defaultStyles]);
    }
    drawKeyShape(model, shapeMap, diffData, diffState) {
        return this.upsertShape('rect', 'keyShape', Object.assign(Object.assign({}, this.mergedStyles.keyShape), { x: this.mergedStyles.keyShape.x -
                this.mergedStyles.keyShape.width / 2, y: this.mergedStyles.keyShape.y -
                this.mergedStyles.keyShape.height / 2 }), shapeMap, model);
    }
    calculateAnchorPosition(keyShapeStyle) {
        const x = convertToNumber(keyShapeStyle.x);
        const y = convertToNumber(keyShapeStyle.y);
        const height = keyShapeStyle.height;
        const width = keyShapeStyle.width;
        const anchorPositionMap = {};
        anchorPositionMap['top'] = [x, y - height / 2];
        anchorPositionMap['left'] = [x - width / 2, y];
        anchorPositionMap['right'] = anchorPositionMap['default'] = [
            x + width / 2,
            y,
        ];
        anchorPositionMap['bottom'] = [x, y + height / 2];
        return anchorPositionMap;
    }
    draw(model, shapeMap, diffData, diffState) {
        const { data = {} } = model;
        let shapes = { keyShape: undefined };
        // keyShape
        shapes.keyShape = this.drawKeyShape(model, shapeMap, diffData);
        // labelShape
        if (data.labelShape) {
            shapes.labelShape = this.drawLabelShape(model, shapeMap, diffData);
        }
        // haloShape
        if (data.haloShape && this.drawHaloShape) {
            shapes.haloShape = this.drawHaloShape(model, shapeMap, diffData);
        }
        // labelBackgroundShape
        if (data.labelBackgroundShape) {
            shapes.labelBackgroundShape = this.drawLabelBackgroundShape(model, shapeMap, diffData);
        }
        // anchor shapes
        if (data.anchorShapes) {
            const anchorShapes = this.drawAnchorShapes(model, shapeMap, diffData, diffState);
            shapes = Object.assign(Object.assign({}, shapes), anchorShapes);
        }
        // iconShape
        if (data.iconShape) {
            shapes.iconShape = this.drawIconShape(model, shapeMap, diffData);
        }
        // badgeShape
        if (data.badgeShapes) {
            const badgeShapes = this.drawBadgeShapes(model, shapeMap, diffData, diffState);
            shapes = Object.assign(Object.assign({}, shapes), badgeShapes);
        }
        // otherShapes
        if (data.otherShapes && this.drawOtherShapes) {
            shapes = Object.assign(Object.assign({}, shapes), this.drawOtherShapes(model, shapeMap, diffData));
        }
        return shapes;
    }
}
//# sourceMappingURL=rect.js.map