var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { groupBy } from '@antv/util';
import { LayerUpdatedCommand } from './layer-updated-command';
import { StateUpdatedCommand } from './state-updated-command';
import { VisibilityUpdatedCommand } from './visibility-updated-command';
import { ItemDataCommand } from './item-data-command';
import { ComboCommand } from './combo-command';
export default class CommandFactory {
    static create(_a) {
        var { type: itemType, action, changes } = _a, rest = __rest(_a, ["type", "action", "changes"]);
        const commands = [];
        const groupedByType = groupBy(changes, 'type');
        for (const type in groupedByType) {
            const itemDataCommandMap = {
                NodeDataUpdated: 'node',
                EdgeDataUpdated: 'edge',
                NodeAdded: 'node',
                EdgeAdded: 'edge',
                NodeRemoved: 'node',
                EdgeRemoved: 'edge',
                TreeStructureChanged: 'combo',
            };
            if (itemDataCommandMap[type]) {
                commands.push(new ItemDataCommand(action, itemType || itemDataCommandMap[type], groupedByType[type], type === 'NodeDataUpdated' ? rest.upsertAncestors : undefined));
            }
        }
        const actionCommandMap = {
            updateState: new StateUpdatedCommand(changes),
            updateVisibility: new VisibilityUpdatedCommand(changes, rest.disableAnimate),
            front: new LayerUpdatedCommand('front', rest.ids),
            back: new LayerUpdatedCommand('back', rest.ids),
            expandCombo: new ComboCommand('expandCombo', rest.ids),
            collapseCombo: new ComboCommand('collapseCombo', rest.ids),
        };
        if (actionCommandMap[action]) {
            commands.push(actionCommandMap[action]);
        }
        return commands;
    }
}
//# sourceMappingURL=command.js.map