import { ITEM_TYPE, ShapeStyle } from '../../../types/item';
import { ComboModel, EdgeModel, ID, IGraph, NodeModel } from '../../../types';
import { Plugin as Base, IPluginBaseConfig } from '../../../types/plugin';
import { ComboLabelPosition } from '../../../types/combo';
import { BubblesetCfg } from './types';
import { HullComponentOptions } from './hullComponent';
interface HullConfig extends IPluginBaseConfig {
    /** Common style for the hulls in this plugin. You can also configure individually for each hulls. */
    style?: ShapeStyle;
    /** Common label config for hulls. You can also configure individually for each hull. */
    labelShape?: ShapeStyle & {
        position?: ComboLabelPosition;
        offsetX?: number;
        offsetY?: number;
        maxWidth?: string | number;
    };
    /** The padding of the hull. */
    padding?: number;
    /** Controlling the effect of the bubble more finely (the scaling degree of the point and edge design, setting the granularity), generally no configuration is required */
    bubbleCfg?: BubblesetCfg;
    /** Initial Hulls. You can also add/remove/update hulls by API of this plugin. */
    hulls?: HullComponentOptions[];
}
export default class Hull extends Base {
    private hullMap;
    private cacheChanges;
    constructor(options?: HullConfig);
    getDefaultCfgs(): HullConfig;
    init(graph: IGraph): void;
    getEvents(): {
        afteritemchange: (param: {
            itemType: ITEM_TYPE;
            action: string;
            models?: (NodeModel | EdgeModel | ComboModel)[];
            ids?: ID[];
        }) => void;
        afterrender: () => void;
    };
    /**
     * viewport change
     * @param param
     */
    protected handleNodesChange(param: {
        itemType: ITEM_TYPE;
        action: string;
        models?: (NodeModel | EdgeModel | ComboModel)[];
        ids?: ID[];
    }): void;
    private debounceUpdateMembers;
    protected handleAfterRender(): void;
    addHull(options: HullComponentOptions | HullComponentOptions[]): void;
    updateHull(options: HullComponentOptions | HullComponentOptions[]): void;
    removeHull(ids: ID | ID[]): void;
    addHullMember(id: ID, members: ID | ID[]): void;
    removeHullMember(id: ID, members: ID | ID[]): void;
    addHullNonMember(id: ID, members: ID | ID[]): void;
    removeHullNonMember(id: ID, members: ID | ID[]): void;
}
export {};
