var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { isArray, throttle, uniqueId } from '@antv/util';
import { Plugin as Base } from '../../../types/plugin';
import { isArrayOverlap } from '../../../util/array';
import { DEFAULT_TEXT_STYLE } from '../../../constant';
import HullComponent from './hullComponent';
export default class Hull extends Base {
    constructor(options) {
        super(options);
        this.hullMap = {};
        this.cacheChanges = {
            remove: [],
            update: [],
        };
        this.debounceUpdateMembers = throttle(() => {
            Object.keys(this.hullMap).forEach((id) => {
                const hullComponent = this.hullMap[id];
                if (this.cacheChanges.remove.length) {
                    isArrayOverlap(hullComponent.options.members, this.cacheChanges.remove) && hullComponent.removeMember(this.cacheChanges.remove);
                }
                else if (this.cacheChanges.update.length) {
                    const modelIds = this.cacheChanges.update.map((model) => model.id);
                    isArrayOverlap(hullComponent.options.members, modelIds) &&
                        hullComponent.updateMembers();
                }
            });
            this.cacheChanges = { update: [], remove: [] };
        }, 50, {
            trailing: true,
            leading: true,
        });
        this.key = (options === null || options === void 0 ? void 0 : options.key) || `hull-${Math.random()}`;
    }
    getDefaultCfgs() {
        return {
            key: `hull-${uniqueId()}`,
            padding: 10,
            style: {
                fill: 'lightblue',
                stroke: 'blue',
                opacity: 0.2,
            },
            labelShape: Object.assign(Object.assign({}, DEFAULT_TEXT_STYLE), { position: 'outside-top', offsetY: -2 }),
            hulls: [],
        };
    }
    init(graph) {
        super.init(graph);
    }
    // class-methods-use-this
    getEvents() {
        return {
            afteritemchange: this.handleNodesChange,
            afterrender: this.handleAfterRender,
        };
    }
    /**
     * viewport change
     * @param param
     */
    handleNodesChange(param) {
        const { itemType, models, ids, action } = param;
        if (itemType === 'edge' || action === 'add')
            return;
        if (action === 'remove') {
            this.cacheChanges.remove = this.cacheChanges.remove.concat(ids);
        }
        else {
            this.cacheChanges.update = this.cacheChanges.update.concat(models);
        }
        this.debounceUpdateMembers();
    }
    handleAfterRender() {
        this.addHull(this.options.hulls);
    }
    addHull(options) {
        const { padding, style, bubbleCfg, labelShape } = this.options;
        const configs = isArray(options) ? options : [options];
        configs.forEach((config) => {
            const { id, style: singleStyle, labelShape: singleLabelShape, members } = config, others = __rest(config, ["id", "style", "labelShape", "members"]);
            const validMembers = members === null || members === void 0 ? void 0 : members.filter((memberId) => this.graph.getNodeData(memberId) || this.graph.getComboData(memberId));
            if (!(validMembers === null || validMembers === void 0 ? void 0 : validMembers.length)) {
                console.warn(`Create hull failed. There are no valid members.`);
                return;
            }
            if (validMembers.length !== members.length) {
                console.warn(`Some member of hull ${id} is not exist in the graph. Hull ${id} is added without those invalid members.`);
            }
            const fullOptions = Object.assign({ id: `${this.key}-${id}`, padding,
                bubbleCfg, style: Object.assign(Object.assign({}, style), singleStyle), labelShape: (singleLabelShape === null || singleLabelShape === void 0 ? void 0 : singleLabelShape.text)
                    ? Object.assign(Object.assign({}, labelShape), singleLabelShape) : undefined, members: validMembers }, others);
            if (this.hullMap[id]) {
                this.updateHull(fullOptions);
                return;
            }
            this.hullMap[id] = new HullComponent(this.graph, fullOptions);
        });
    }
    updateHull(options) {
        const configs = isArray(options) ? options : [options];
        configs.forEach((config) => {
            const { id } = config, others = __rest(config, ["id"]);
            if (!this.hullMap[id]) {
                console.warn(`Update hull component failed. The bubble with id ${id} is not existed.`);
                return;
            }
            this.hullMap[id].updateOptions(others);
        });
    }
    removeHull(ids) {
        const idArr = isArray(ids) ? ids : [ids];
        idArr.forEach((id) => {
            const hullComponent = this.hullMap[id];
            hullComponent.destroy();
            delete this.hullMap[id];
        });
    }
    addHullMember(id, members) {
        const hullComponent = this.hullMap[id];
        if (!hullComponent) {
            console.warn(`Add member to hull faield. The hull with id ${id} is not exist`);
            return;
        }
        hullComponent.addMember(members);
    }
    removeHullMember(id, members) {
        const hullComponent = this.hullMap[id];
        if (!hullComponent) {
            console.warn(`Remove member from hull faield. The hull with id ${id} is not exist`);
            return;
        }
        hullComponent.removeMember(members);
    }
    addHullNonMember(id, members) {
        const hullComponent = this.hullMap[id];
        if (!hullComponent) {
            console.warn(`Add non member to hull faield. The hull with id ${id} is not exist`);
            return;
        }
        hullComponent.addNonMember(members);
    }
    removeHullNonMember(id, members) {
        const hullComponent = this.hullMap[id];
        if (!hullComponent) {
            console.warn(`Remove non member from hull faield. The hull with id ${id} is not exist`);
            return;
        }
        hullComponent.removeNonMember(members);
    }
}
//# sourceMappingURL=index.js.map