import { isArray } from '@antv/util';
import { mergeStyles } from '../../util/shape';
import { BaseThemeSolver } from './base';
export class SpecThemeSolver extends BaseThemeSolver {
    solver(options, themes) {
        const { base = 'light', specification } = options;
        const baseSpec = themes[base];
        const mergedSpec = Object.assign({}, baseSpec);
        if (specification) {
            ['node', 'edge', 'combo'].forEach((itemType) => {
                if (!specification[itemType])
                    return;
                const { palette = mergedSpec[itemType].palette, lodStrategy = mergedSpec[itemType].lodStrategy, dataTypeField, } = specification[itemType];
                let { getStyleSets } = specification[itemType];
                if (dataTypeField && !getStyleSets) {
                    getStyleSets = (paletteProps) => {
                        return paletteProps.map((color) => ({
                            default: {
                                keyShape: itemType === 'edge' ? { stroke: color } : { fill: color },
                            },
                        }));
                    };
                }
                // merge the custom part spec and the built-in spec
                const { styles: [baseStyles], } = baseSpec[itemType];
                const incomingStyles = (getStyleSets === null || getStyleSets === void 0 ? void 0 : getStyleSets(palette)) || {};
                let mergedStyles;
                if (isArray(incomingStyles)) {
                    mergedStyles = incomingStyles.map((incomingStyle) => {
                        const mergedStatesStyles = Object.assign({}, baseStyles);
                        Object.keys(incomingStyle).forEach((stateName) => {
                            mergedStatesStyles[stateName] = mergeStyles([
                                baseStyles[stateName],
                                incomingStyle[stateName],
                            ]);
                        });
                        return mergedStatesStyles;
                    });
                }
                else {
                    mergedStyles = { others: baseStyles };
                    Object.keys(incomingStyles).forEach((dataType) => {
                        const mergedStatesStyles = Object.assign({}, baseStyles);
                        const incomintStyle = incomingStyles[dataType];
                        Object.keys(incomintStyle).forEach((stateName) => {
                            mergedStatesStyles[stateName] = mergeStyles([
                                baseStyles[stateName],
                                incomintStyle[stateName],
                            ]);
                        });
                        mergedStyles[dataType] = mergedStatesStyles;
                    });
                }
                mergedSpec[itemType] = {
                    dataTypeField,
                    palette,
                    lodStrategy,
                    styles: mergedStyles,
                };
            });
            mergedSpec['canvas'] = Object.assign(Object.assign({}, baseSpec.canvas), specification.canvas);
        }
        return mergedSpec;
    }
}
//# sourceMappingURL=spec.js.map