import { IElement } from '@antv/g';
import { EdgeAdded, EdgeDataUpdated, EdgeRemoved, EdgeUpdated, ID, NodeAdded, NodeDataUpdated, NodeRemoved, TreeStructureChanged } from '@antv/graphlib';
import { IG6GraphEvent, IGraph, NodeModelData } from '../types';
import { GraphCore } from '../types/data';
import { EdgeModelData } from '../types/edge';
export type ItemInfo = {
    itemType: 'canvas' | 'node' | 'edge' | 'combo';
    itemId: ID | undefined;
    groupElement: IElement;
};
/**
 * Given an element, which might be a child shape of a node/edge,
 * get its belonging item.
 */
export declare const getItemInfoFromElement: (element: IElement) => ItemInfo | null;
/**
 * Returns the event props for graph's contextmenu event.
 * @param event Original g6 event props.
 * @param graph Graph instance.
 * @returns Contextmenu event props.
 */
export declare const getContextMenuEventProps: (event: IG6GraphEvent, graph: IGraph) => IG6GraphEvent;
export type GroupedChanges = {
    NodeRemoved: NodeRemoved<NodeModelData>[];
    EdgeRemoved: EdgeRemoved<EdgeModelData>[];
    NodeAdded: NodeAdded<NodeModelData>[];
    EdgeAdded: EdgeAdded<EdgeModelData>[];
    NodeDataUpdated: NodeDataUpdated<NodeModelData>[];
    EdgeUpdated: EdgeUpdated<EdgeModelData>[];
    EdgeDataUpdated: EdgeDataUpdated<EdgeModelData>[];
    TreeStructureChanged: TreeStructureChanged[];
    ComboStructureChanged: TreeStructureChanged[];
};
/**
 * Group the changes with change types from graphCore's changes.
 * @param graphCore
 * @param changes
 * @returns
 */
export declare const getGroupedChanges: (graphCore: GraphCore, changes: any) => GroupedChanges;
