import { getCombinedBoundsByItem } from './shape';
/**
 * Find the edges whose source and target are both in the ids.
 * @param graph graph instance
 * @param ids the end nodes/combos id list
 * @returns id list of the edges meet the condition
 */
export const getEdgesBetween = (graph, ids) => {
    const edgeIdSet = new Set();
    ids.forEach((endId) => {
        const edgesData = graph.getRelatedEdgesData(endId);
        edgesData.forEach((edge) => {
            const { source, target, id } = edge;
            if (!graph.getItemVisible(id))
                return; // hidden edge is not selectable
            if (ids.includes(source) && ids.includes(target))
                edgeIdSet.add(id);
        });
    });
    return Array.from(edgeIdSet);
};
export const upsertTransientItem = (item, nodeGroup, edgeGroup, comboGroup, transientItemMap, itemMap, graphCore, onlyDrawKeyShape, upsertAncestors = true) => {
    let transientItem = transientItemMap.get(item.model.id);
    if (transientItem) {
        return transientItem;
    }
    if (item.type === 'node') {
        transientItem = item.clone(nodeGroup, onlyDrawKeyShape, true);
    }
    else if (item.type === 'edge') {
        const source = upsertTransientItem(item.sourceItem, nodeGroup, edgeGroup, comboGroup, transientItemMap, itemMap, graphCore, onlyDrawKeyShape, false);
        const target = upsertTransientItem(item.targetItem, nodeGroup, edgeGroup, comboGroup, transientItemMap, itemMap, graphCore, onlyDrawKeyShape, false);
        transientItem = item.clone(edgeGroup, source, target, undefined, true);
    }
    else {
        // find the succeeds to upsert transients
        const childItems = [];
        const children = graphCore.getChildren(item.model.id, 'combo');
        children.forEach((childModel) => {
            const childItem = itemMap.get(childModel.id);
            if (childItem) {
                childItems.push(upsertTransientItem(childItem, nodeGroup, edgeGroup, comboGroup, transientItemMap, itemMap, graphCore, onlyDrawKeyShape, false));
            }
        });
        transientItem = item.clone(comboGroup, onlyDrawKeyShape, true, () => getCombinedBoundsByItem(childItems), // getCombinedBounds
        () => childItems);
        transientItem.toBack();
    }
    transientItemMap.set(item.model.id, transientItem);
    transientItem.transient = true;
    if (item.type !== 'edge' &&
        upsertAncestors &&
        graphCore.hasTreeStructure('combo')) {
        // find the ancestors to upsert transients
        let currentAncestor = graphCore.getParent(item.model.id, 'combo');
        while (currentAncestor) {
            const ancestorItem = itemMap.get(currentAncestor.id);
            if (ancestorItem) {
                const transientAncestor = upsertTransientItem(ancestorItem, nodeGroup, edgeGroup, comboGroup, transientItemMap, itemMap, graphCore, onlyDrawKeyShape);
                transientAncestor.toBack();
            }
            currentAncestor = graphCore.getParent(currentAncestor.id, 'combo');
        }
    }
    return transientItem;
};
//# sourceMappingURL=item.js.map