import { Tuple3Number } from '@antv/g';
import { vec2 } from '@antv/matrix-util';
import { LoopPosition } from '../types/loop';
/**
 * Calculates the start and end points of an ellipse based on the given position.
 * @param {LoopPosition} position - The position of the ellipse (top, top-right, right, bottom-right, bottom, bottom-left, left, top-left).
 * @param {Tuple3Number} center - The center coordinates.
 * @param {number} halfOfWidth - Half of the width.
 * @param {number} halfOfHeight - Half of the height.
 * @returns {[vec2, vec2]} - An array containing the start and end points of the ellipse.
 */
export declare const calculatePointForEllipse: (position: LoopPosition, center: Tuple3Number, halfOfWidth: number, halfOfHeight: number) => [vec2, vec2];
/**
 * Calculates the start and end points of a shape other than an ellipse based on the given position.
 * @param {LoopPosition} position
 * @param {Tuple3Number} center - The center coordinates.
 * @param {number} halfOfWidth - Half of the width of keyShape.
 * @param {number} halfOfHeight - Half of the height of keyShape.
 * @param {number} customPointPadding - Optional custom padding for the point calculation.
 */
export declare const calculatePointForOtherShapes: (position: LoopPosition, center: Tuple3Number, halfOfWidth: number, halfOfHeight: number, customPointPadding?: number) => [vec2, vec2];
/**
 * Calculate the point on the ellipse at the given angle.
 * @param {number} halfOfWidth - Half of the width of the ellipse.
 * @param {number} halfOfHeight - Half of the height of the ellipse.
 * @param {number} angleInRadians - The angle in radians.
 * @returns {vec2} - The point on the ellipse at the given angle.
 */
export declare const getPointOnEllipseAtAngle: (halfOfWidth: number, halfOfHeight: number, angleInRadians: number) => vec2;
/**
 * Calculates an array of points on an ellipse at the given angles.
 * @param {number} halfOfWidth - Half of the width of the ellipse.
 * @param {number} halfOfHeight - Half of the height of the ellipse.
 * @param {number} startAngle - The starting angle in radians.
 * @param {number} endAngle - The ending angle in radians.
 */
export declare const getPointsOnEllipseAtAngles: (halfOfWidth: number, halfOfHeight: number, startAngle: number, endAngle: number) => number[];
