import { vec2 } from '@antv/matrix-util';
import { Point } from '../types/common';
/**
 * Find the nearest on in points to the curPoint.
 * @param points
 * @param curPoint
 * @returns
 */
export declare const getNearestPoint: (points: Point[], curPoint: Point) => {
    index: number;
    nearestPoint: Point;
};
/**
 * Get distance by two points.
 * @param {vec2} p1 first point
 * @param {vec2} p2 second point
 */
export declare const distanceVec: (p1: vec2, p2: vec2) => number;
/**
 * Get distance by two points.
 * @param {Point} p1 first point
 * @param {Point} p2 second point
 */
export declare const distance: (p1: Point, p2: Point) => number;
/**
 * If the p1 and p2 are the same.
 * @param p1
 * @param p2
 * @returns
 */
export declare const isSamePoint: (p1: Point, p2: Point) => boolean;
/**
 * Get point and circle intersect point.
 * @param {ICircle} circle Circle's center x,y and radius r
 * @param {Point} point Point x,y
 * @return {Point} calculated intersect point
 */
export declare const getCircleIntersectByPoint: (circleProps: {
    x: number;
    y: number;
    r: number;
}, point: Point) => Point | null;
/**
 * Get point and ellipse inIntersect.
 * @param {Object} ellipse ellipse center x,y and radius rx,ry
 * @param {Object} point Point x,y
 * @return {object} calculated intersect point
 */
export declare const getEllipseIntersectByPoint: (ellipseProps: {
    rx: number;
    ry: number;
    x: number;
    y: number;
}, point: Point) => Point;
/**
 * Point and rectangular intersection point.
 * @param  {IRect} rect  rect
 * @param  {Point} point point
 * @return {PointPoint} rst;
 */
export declare const getRectIntersectByPoint: (rectProps: {
    x: number;
    y: number;
    width: number;
    height: number;
}, point: Point) => Point | null;
/**
 * Get the intersect point of two lines.
 * @param  {Point}  p0 The start point of the first line.
 * @param  {Point}  p1 The end point of the first line.
 * @param  {Point}  p2 The start point of the second line.
 * @param  {Point}  p3 The end point of the second line.
 * @return {Point}  Calculated intersect point.
 */
export declare const getLineIntersect: (p0: Point, p1: Point, p2: Point, p3: Point) => Point | null;
/**
 * Determine if three points are bending (not lie on a straight line)
 * @param p0 the first 2d point
 * @param p1 the second 2d point
 * @param p2 the third 2d point
 * @returns
 */
export declare const isBending: (p0: Point, p1: Point, p2: Point) => boolean;
