import { Circle, DisplayObject, Ellipse, Group, IElement, Line, Polygon, Polyline, Rect, Text, Image, Path, AABB, Tuple3Number } from '@antv/g';
import { Point } from '../types/common';
import { EdgeDisplayModel, EdgeShapeMap } from '../types/edge';
import { GShapeStyle, SHAPE_TYPE, ItemShapeStyles, ShapeStyle, SHAPE_TYPE_3D } from '../types/item';
import { NodeDisplayModel, NodeShapeMap } from '../types/node';
import { ComboDisplayModel, IGraph } from '../types';
import Node from '../item/node';
import Edge from '../item/edge';
import Combo from '../item/combo';
export declare const ShapeTagMap: {
    circle: typeof Circle;
    rect: typeof Rect;
    ellipse: typeof Ellipse;
    polygon: typeof Polygon;
    line: typeof Line;
    polyline: typeof Polyline;
    text: typeof Text;
    image: typeof Image;
    path: typeof Path;
};
export declare const LOCAL_BOUNDS_DIRTY_FLAG_KEY = "data-item-local-bounds-dirty";
export declare const createShape: (type: SHAPE_TYPE, style: GShapeStyle, id: string) => Path | Line | Polygon | Rect | Circle | Ellipse | Text | Image;
/**
 * Create (if does not exit in shapeMap) or update the shape according to the configurations.
 * @param type shape's type
 * @param id unique string to indicates the shape
 * @param style style to be updated
 * @param shapeMap the shape map of a node / edge / combo
 * @param model data model of the node / edge / combo
 * @returns
 */
export declare const upsertShape: (type: SHAPE_TYPE, id: string, style: GShapeStyle, shapeMap: {
    [shapeId: string]: DisplayObject<any, any>;
}, model?: NodeDisplayModel | EdgeDisplayModel | ComboDisplayModel) => DisplayObject;
export declare const getGroupSucceedMap: (group: IElement, map?: {
    [id: string]: IElement<any, any>;
}) => {
    [id: string]: IElement<any, any>;
};
/**
 * Update shapes in the intersaction of prevShapeMap and newShapeMap;
 * Remove shapes in the prevShapeMap - newShapeMap (if removeDiff is true);
 * Add shapes in the newShapeMap - prevShapeMap;
 * @param prevShapeMap previous shape map
 * @param newShapeMap new shape map
 * @param group container group
 * @returns merged shape map
 */
export declare const updateShapes: (prevShapeMap: {
    [id: string]: DisplayObject<any, any>;
}, newShapeMap: {
    [id: string]: DisplayObject<any, any>;
}, group: Group, removeDiff?: boolean, shouldUpdate?: (id: string) => boolean) => NodeShapeMap | EdgeShapeMap;
/**
 * Format the number or array padding to an array with length 4, [padding-top, padding-right, padding-bottom, padding-left].
 * @param value value to be formatted
 * @param defaultArr default value
 * @returns [padding-top, padding-right, padding-bottom, padding-left]
 */
export declare const formatPadding: (value: any, defaultArr?: number[]) => any[];
/**
 * Merge multiple shape style map including undefined value in incoming map.
 * @param styleMaps shapes' styles map array, the latter item in the array will be merged into the former
 * @returns
 */
export declare const mergeStyles: (styleMaps: ItemShapeStyles[]) => ItemShapeStyles;
/**
 * Whether two polygons are intersected.
 * @param points1 vertex array of polygon1
 * @param points2 vertex array of polygon2
 */
export declare const isPolygonsIntersect: (points1: number[][], points2: number[][]) => boolean;
export declare const intersectBBox: (box1: Partial<AABB>, box2: Partial<AABB>) => boolean;
/**
 * Whether point is inside the polygon (ray algo)
 * @param points
 * @param x
 * @param y
 */
export declare const isPointInPolygon: (points: number[][], x: number, y: number) => boolean;
/**
 * Get the intersect point of two line.
 * @param  {Point}  p0 start of the first line
 * @param  {Point}  p1 end of the first line
 * @param  {Point}  p2 start of the second line
 * @param  {Point}  p3 end of the second line
 * @return {Point}  the intersect point
 */
export declare const getLineIntersect: (p0: Point, p1: Point, p2: Point, p3: Point) => Point | null;
/**
 * Will the fields in style affect the bbox.
 * @param type shape type
 * @param style style object
 * @returns
 */
export declare const isStyleAffectBBox: (type: SHAPE_TYPE | SHAPE_TYPE_3D, style: ShapeStyle) => boolean;
/**
 * Estimate the width of the shape according to the given style.
 * @param shape target shape
 * @param style computed merged style
 * @param bounds shape's local bounds
 * @returns
 */
export declare const getShapeLocalBoundsByStyle: (shape: DisplayObject, style: ShapeStyle, bbox?: AABB) => {
    min: number[];
    max: number[];
    center: number[];
};
/**
 * Combine multiple bounds into one.
 * @param bounds bounds' array
 * @returns combined bounds
 */
export declare const combineBounds: (bounds: {
    min: number[];
    max: number[];
    center: number[];
}[]) => {
    min: number[];
    max: number[];
    center: number[];
};
/**
 * Calculate the combined bounds according to the sub data models.
 * @param graph graph instance
 * @param models sub data models
 * @returns combined bounds
 */
export declare const getCombinedBoundsByData: (graph: IGraph, models: (Node | Combo)[]) => {
    center: Tuple3Number;
    min: Tuple3Number;
    max: Tuple3Number;
    size: Tuple3Number;
} | false;
export declare const getCombinedBoundsByItem: (items: (Node | Combo | Edge)[]) => false | {
    center: Tuple3Number;
    min: Tuple3Number;
    max: Tuple3Number;
    size: Tuple3Number;
};
