"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const g_1 = require("@antv/g");
const util_1 = require("@antv/util");
const shape_1 = require("../util/shape");
const animate_1 = require("../util/animate");
const point_1 = require("../util/point");
const item_1 = __importDefault(require("./item"));
class Edge extends item_1.default {
    constructor(props) {
        super(props);
        this.destroyed = false;
        /** Set to different value in implements */
        this.type = 'edge';
        /**
         * Sometimes no changes on edge data, but need to re-draw it
         * e.g. source and target nodes' position changed
         */
        this.forceUpdate = (0, util_1.throttle)(() => {
            if (!this.destroyed)
                this.draw(this.displayModel);
        }, 16, {
            leading: true,
            trailing: true,
        });
        this.init(Object.assign(Object.assign({}, props), { type: 'edge' }));
        const { sourceItem, targetItem, nodeMap } = props;
        this.sourceItem = sourceItem;
        this.targetItem = targetItem;
        this.nodeMap = nodeMap;
        // todo: combo
        if (sourceItem.getType() === 'node') {
            this.nodeMap.set(sourceItem.getID(), sourceItem);
        }
        if (sourceItem.getType() === 'node') {
            this.nodeMap.set(targetItem.getID(), targetItem);
        }
        this.draw(this.displayModel);
    }
    draw(displayModel, diffData, diffState, animate = true, onfinish = () => { }) {
        var _a, _b, _c;
        // get the point near the other end
        const { sourceAnchor, targetAnchor, keyShape } = displayModel.data;
        const sourcePosition = this.sourceItem.getPosition();
        const targetPosition = this.targetItem.getPosition();
        let targetPrevious = sourcePosition;
        let sourcePrevious = targetPosition;
        // TODO: type
        // @ts-ignore
        if ((_a = keyShape === null || keyShape === void 0 ? void 0 : keyShape.controlPoints) === null || _a === void 0 ? void 0 : _a.length) {
            // @ts-ignore
            const controlPointsBesideEnds = keyShape.controlPoints.filter((point) => !(0, point_1.isSamePoint)(point, sourcePosition) &&
                !(0, point_1.isSamePoint)(point, targetPosition));
            sourcePrevious = (0, point_1.getNearestPoint)(controlPointsBesideEnds, sourcePosition).nearestPoint;
            targetPrevious = (0, point_1.getNearestPoint)(controlPointsBesideEnds, targetPosition).nearestPoint;
        }
        const sourcePoint = this.sourceItem.getAnchorPoint(sourcePrevious, sourceAnchor);
        const targetPoint = this.targetItem.getAnchorPoint(targetPrevious, targetAnchor);
        this.renderExt.mergeStyles(displayModel);
        const firstRendering = !((_b = this.shapeMap) === null || _b === void 0 ? void 0 : _b.keyShape);
        this.renderExt.setSourcePoint(sourcePoint);
        this.renderExt.setTargetPoint(targetPoint);
        this.renderExt.setNodeMap(this.nodeMap);
        const shapeMap = this.renderExt.draw(displayModel, sourcePoint, targetPoint, this.shapeMap, diffData, diffState);
        // add shapes to group, and update shapeMap
        this.shapeMap = (0, shape_1.updateShapes)(this.shapeMap, shapeMap, this.group);
        // handle shape's and group's animate
        const { animates, disableAnimate } = displayModel.data;
        const usingAnimates = Object.assign({}, animates);
        const targetStyles = this.renderExt.mergedStyles;
        const { haloShape, labelShape } = this.shapeMap;
        haloShape === null || haloShape === void 0 ? void 0 : haloShape.toBack();
        labelShape === null || labelShape === void 0 ? void 0 : labelShape.toFront();
        super.draw(displayModel, diffData, diffState);
        this.renderExt.updateCache(this.shapeMap);
        if (firstRendering) {
            // update the transform
            this.renderExt.onZoom(this.shapeMap, this.zoom);
        }
        else {
            // terminate previous animations
            this.stopAnimations();
        }
        const timing = firstRendering ? 'buildIn' : 'update';
        // handle shape's animate
        if (animate && !disableAnimate && ((_c = usingAnimates[timing]) === null || _c === void 0 ? void 0 : _c.length)) {
            this.animations = (0, animate_1.animateShapes)(usingAnimates, targetStyles, // targetStylesMap
            this.shapeMap, // shapeMap
            this.group, firstRendering ? 'buildIn' : 'update', this.changedStates, this.animateFrameListener, (canceled) => onfinish(displayModel.id, canceled));
        }
        if (firstRendering && !this.visible) {
            this.visible = true;
            this.hide(false);
        }
    }
    /**
     * Update end item for item and re-draw the edge
     * @param type update source or target
     * @param endItem new item to ended
     */
    updateEnd(type, endItem) {
        if (type === 'source')
            this.sourceItem = endItem;
        else if (type === 'target')
            this.targetItem = endItem;
        this.draw(this.displayModel);
    }
    // public update(model: EdgeModel) {
    //   super.update(model);
    // }
    clone(containerGroup, sourceItem, targetItem, onlyKeyShape, disableAnimate) {
        if (onlyKeyShape) {
            const clonedKeyShape = this.shapeMap.keyShape.cloneNode();
            const clonedGroup = new g_1.Group();
            clonedGroup.appendChild(clonedKeyShape);
            containerGroup.appendChild(clonedGroup);
            return clonedGroup;
        }
        const clonedModel = (0, util_1.clone)(this.model);
        clonedModel.data.disableAnimate = disableAnimate;
        const clonedEdge = new Edge({
            model: clonedModel,
            renderExtensions: this.renderExtensions,
            sourceItem,
            targetItem,
            nodeMap: this.nodeMap,
            containerGroup,
            mapper: this.mapper,
            stateMapper: this.stateMapper,
            zoom: this.zoom,
            theme: {
                styles: this.themeStyles,
                lodStrategy: this.lodStrategy,
            },
        });
        Object.keys(this.shapeMap).forEach((shapeId) => {
            var _a;
            if (!this.shapeMap[shapeId].isVisible())
                (_a = clonedEdge.shapeMap[shapeId]) === null || _a === void 0 ? void 0 : _a.hide();
        });
        return clonedEdge;
    }
}
exports.default = Edge;
//# sourceMappingURL=edge.js.map