import { ID } from '@antv/graphlib';
import { ComboModel, IGraph } from '../../types';
import { GraphCore } from '../../types/data';
import { EdgeModel, EdgeUserModelData } from '../../types/edge';
import { ITEM_TYPE } from '../../types/item';
import { NodeModel, NodeUserModelData } from '../../types/node';
/**
 * Manages the data transform extensions;
 * Storages user data and inner data.
 */
export declare class DataController {
    graph: IGraph;
    extensions: any[];
    /**
     * User input data.
     */
    userGraphCore: GraphCore;
    /**
     * Inner data stored in graphCore structure.
     */
    graphCore: GraphCore;
    /**
     * A flag to note whether the tree data structure should be recalculated and establish.
     */
    private treeDirtyFlag;
    constructor(graph: IGraph<any, any>);
    findData(type: ITEM_TYPE, condition: ID[] | Function): EdgeModel[] | NodeModel[] | ComboModel[];
    findAllData(type: ITEM_TYPE): EdgeModel[] | NodeModel[] | ComboModel[];
    findRelatedEdges(nodeId: ID, direction?: 'in' | 'out' | 'both'): import("@antv/graphlib").Edge<EdgeUserModelData>[];
    findNeighborNodes(nodeId: ID, direction?: 'in' | 'out' | 'both'): import("@antv/graphlib").Node<NodeUserModelData>[];
    findChildren(comboId: ID, treeKey: string): import("@antv/graphlib").Node<NodeUserModelData>[];
    /**
     * Subscribe the lifecycle of graph.
     */
    private tap;
    /**
     * Get the extensions from useLib.
     */
    private getExtensions;
    /**
     * Listener of graph's datachange hook.
     * @param param contains new graph data and type of data change
     */
    private onDataChange;
    private onTreeCollapseExpand;
    private onLayout;
    private onGraphInit;
    private onBehaviorChange;
    private onModeChange;
    private establishUserGraphCoreTree;
    /**
     * Change data by replace, merge repalce, or union.
     * @param data new data
     * @param changeType type of data change, 'replace' means discard the old data. 'mergeReplace' means merge the common part. 'union' means merge whole sets of old and new one
     */
    private changeData;
    /**
     * Remove part of old data.
     * @param data data to be removed which is part of old one
     */
    private removeData;
    /**
     * Update part of old data.
     * @param data data to be updated which is part of old one
     */
    private updateData;
    private formatData;
    /**
     * Relatively move the position of combo a dinstance (dx and dy in combo data).
     * @param data graph data which contains the combo dx dy info
     */
    private moveCombo;
    private addCombo;
    /**
     * Update graphCore with transformed userGraphCore data.
     */
    private updateGraphCore;
    /**
     * Clone data from userGraphCore, and run transforms
     * @returns transformed data and the id map list
     */
    private transformData;
    /**
     * convert and store tree structure to graphCore
     * @param dataType
     * @param data
     */
    private updateTreeGraph;
}
