import { IGraph } from '../../types';
/**
 * Manages the interaction extensions and graph modes;
 * Storage related data.
 */
export declare class InteractionController {
    private graph;
    private mode;
    /**
     * Available behaviors of current mode.
     * @example
     * { 'drag-node': DragNode, 'drag-canvas': DragCanvas }
     */
    private behaviorMap;
    /**
     * Listeners added by all current behaviors.
     * @example
     * {
     *   'drag-node': { 'node:pointerEnter': function },
     * }
     */
    private listenersMap;
    private prevItemInfo;
    constructor(graph: IGraph<any, any>);
    /**
     * Subscribe the lifecycle of graph.
     */
    private tap;
    private validateMode;
    private initBehavior;
    private destroyBehavior;
    private addListeners;
    private removeListeners;
    /**
     * Listener of graph's init hook. Add listeners from behaviors to graph.
     * @param param contains the mode to switch to
     */
    private onModeChange;
    getMode(): string;
    /**
     * Listener of graph's behaviorchange hook. Update, add, or remove behaviors from modes.
     * @param param contains action, modes, and behaviors
     */
    private onBehaviorChange;
    private initEvents;
    private handleCanvasEvent;
    /**
     * Emit item's pointerleave/pointerenter events when pointer moves on the canvas.
     */
    private handlePointerMove;
    private handleDOMEvent;
}
