import { AABB, DisplayObject } from '@antv/g';
import { ID } from '@antv/graphlib';
import { IGraph } from '../../types';
import { GraphCore } from '../../types/data';
import Node from '../../item/node';
import Edge from '../../item/edge';
import Combo from '../../item/combo';
import { ITEM_TYPE } from '../../types/item';
import { BaseNode } from '../../stdlib/item/node/base';
import { BaseEdge } from '../../stdlib/item/edge/base';
/**
 * Manages and stores the node / edge / combo items.
 */
export declare class ItemController {
    graph: IGraph;
    nodeExtensions: BaseNode[];
    edgeExtensions: BaseEdge[];
    comboExtensions: BaseNode[];
    zoom: number;
    /**
     * Node / edge / combo items map
     */
    private itemMap;
    /**
     * node / edge / combo 's mapper in graph config
     */
    private nodeMapper;
    private edgeMapper;
    private comboMapper;
    private nodeStateMapper;
    private edgeStateMapper;
    private comboStateMapper;
    private nodeGroup;
    private edgeGroup;
    private comboGroup;
    private transientNodeGroup;
    private transientEdgeGroup;
    private transientComboGroup;
    private nodeDataTypeSet;
    private edgeDataTypeSet;
    private comboDataTypeSet;
    private transientObjectMap;
    private transientItemMap;
    constructor(graph: IGraph<any, any>);
    /**
     * Subscribe the lifecycle of graph.
     */
    private tap;
    /**
     * Get the extensions from useLib, stdLib is a subset of useLib.
     */
    private getExtensions;
    /**
     * Listener of runtime's render hook.
     * @param param contains inner data stored in graphCore structure
     */
    private onRender;
    /**
     * Listener of runtime's itemchange lifecycle hook.
     * @param param
     */
    private onChange;
    /**
     * The listener for item state changing.
     * @param param
     * {
     *   ids: ids of the items to be set state
     *   states: state names to set
     *   value: state value
     * }
     */
    private onItemStateChange;
    private onItemVisibilityChange;
    private onItemZIndexChange;
    private onViewportChange;
    private onThemeChange;
    private onDestroy;
    private onTransientUpdate;
    getTransient(id: string): DisplayObject<any, any>;
    getTransientItem(id: ID): any;
    findDisplayModel(id: ID): import("../../types/item").ItemDisplayModel;
    /**
     * Create nodes with inner data to canvas.
     * @param models nodes' inner datas
     */
    private renderNodes;
    private renderCombos;
    /**
     * Create edges with inner data to canvas.
     * @param models edges' inner datas
     */
    private renderEdges;
    /**
     * Get the id of the item which have the state with true value
     * @param itemType item's type
     * @param state state name
     * @param value state value, true by default
     * @returns
     */
    findIdByState(itemType: ITEM_TYPE, state: string, value?: string | boolean): ID[];
    /**
     * Get the state value for the item with id
     * @param id item' id
     * @param state state name
     * @returns {boolean | string} the state value
     */
    getItemState(id: ID, state: string): string | boolean;
    getItemAllStates(id: ID): string[];
    getItemById(id: ID): Node | Combo | Edge;
    getItemBBox(id: ID, isKeyShape?: boolean, isTransient?: boolean): AABB | false;
    getItemVisible(id: ID): boolean;
    /**
     * Identify edges that are intersected by a particular node
     * @param nodeId node id
     * @param graphCore
     * @returns
     */
    findNearEdgesByNode(nodeId: ID, graphCore: GraphCore): import("@antv/graphlib").Edge<import("../../types/edge").EdgeUserModelData>[];
    sortByComboTree(graphCore: GraphCore): void;
    private collapseCombo;
    private expandCombo;
    /**
     * Collapse or expand a sub tree according to action
     * @param params
     */
    private onTreeCollapseExpand;
    /**
     * Collapse sub tree(s).
     * @param rootModels The root node models of sub trees
     * @param graphCore
     * @param animate Whether enable animations for expanding, true by default
     * @returns
     */
    private collapseSubTree;
    /**
     * Expand sub tree(s).
     * @param rootModels The root node models of sub trees.
     * @param graphCore
     * @param animate Whether enable animations for expanding, true by default.
     * @returns
     */
    private expandSubTree;
}
