import { IGraph } from '../../types';
import { Plugin as PluginBase } from '../../types/plugin';
/**
 * Manages free plugin extensions and graph layout.
 */
export declare class PluginController {
    extensions: any;
    graph: IGraph;
    /**
     * Plugins on graph.
     * @example
     * { 'minimap': Minimap, 'tooltip': Tooltip }
     */
    private pluginMap;
    /**
     * Listeners added by all current plugins.
     * @example
     * {
     *   'minimap': { 'afterlayout': function },
     * }
     */
    private listenersMap;
    constructor(graph: IGraph<any, any>);
    /**
     * Subscribe the lifecycle of graph.
     */
    private tap;
    private onPluginInit;
    private initPlugin;
    private onPluginChange;
    /**
     * Check if a plugin with the specified plugin key exists.
     * @param {string} pluginKey The key of the plugin to check.
     */
    hasPlugin(pluginKey: string): boolean;
    /**
     * Retrieve the plugin with the specified plugin key.
     * @param {string} pluginKey The key of the plugin to check.
     */
    getPlugin(pluginKey: string): PluginBase;
    private addListeners;
    private removeListeners;
    private onDestroy;
    destroy(): void;
}
