import type { ID, IG6GraphEvent } from '../../types';
import { Behavior } from '../../types/behavior';
declare const MOUSE_TRIGGERS: readonly ["mouseenter", "click"];
type Trigger = (typeof MOUSE_TRIGGERS)[number];
export interface ActivateRelationsOptions {
    /**
     * Whether to allow multiple selection.
     * Defaults to true.
     * If set to false, `trigger` options will be ignored.
     */
    multiple?: boolean;
    /**
     * The key to pressed with mouse click to apply multiple selection.
     * Defaults to `"click"`.
     * Could be "click", "mouseenter".
     */
    trigger?: Trigger;
    /**
     *
     * Defaults to `"selected"`.
     *
     */
    activeState?: 'selected';
    /**
     * Whether allow the behavior happen on the current item.
     */
    shouldBegin?: (event: IG6GraphEvent) => boolean;
    /**
     * Whether to update item state.
     * If it returns false, you may probably listen to `eventName` and
     * manage states or data manually
     */
    shouldUpdate?: (event: IG6GraphEvent) => boolean;
}
export declare class ActivateRelations extends Behavior {
    timer: number;
    inactiveItems: {};
    prevNodeIds: ID[];
    prevEdgeIds: ID[];
    constructor(options: Partial<ActivateRelationsOptions>);
    getEvents: () => {
        'node:mouseenter': (e: IG6GraphEvent) => void;
        'combo:mouseenter': (e: IG6GraphEvent) => void;
        'node:mouseleave': (e: any) => void;
        'combo:mouseleave': (e: any) => void;
        'node:click'?: undefined;
        'combo:click'?: undefined;
        'canvas:click'?: undefined;
    } | {
        'node:click': (e: IG6GraphEvent) => void;
        'combo:click': (e: IG6GraphEvent) => void;
        'canvas:click': (e: any) => void;
        'node:mouseenter'?: undefined;
        'combo:mouseenter'?: undefined;
        'node:mouseleave'?: undefined;
        'combo:mouseleave'?: undefined;
    };
    setAllItemStates(e: IG6GraphEvent): void;
    clearActiveState(e: any): void;
}
export {};
