import { DisplayObject } from '@antv/g';
import { ID } from '@antv/graphlib';
import { Behavior } from '../../types/behavior';
import { Point } from '../../types/common';
import { IG6GraphEvent } from '../../types/event';
import { ITEM_TYPE } from '../../types/item';
declare const ALLOWED_TRIGGERS: readonly ["drag", "shift", "ctrl", "alt", "meta"];
type Trigger = (typeof ALLOWED_TRIGGERS)[number];
type IDSet = {
    nodes: ID[];
    edges: ID[];
    combos: ID[];
};
export interface BrushSelectOptions {
    /**
     * The key to pressed with mouse click to apply multiple selection.
     * Defaults to `"shift"`.
     * Could be "drag", "shift", "ctrl", "alt", or "meta".
     */
    trigger: Trigger;
    /**
     * Item types to be able to select.
     * Defaults to `["nodes"]`.
     * Should be an array of "node", "edge", or "combo".
     */
    itemTypes: Array<'node' | 'edge' | 'combo'>;
    /**
     * The state to be applied when select.
     * Defaults to `"selected"`.
     * Can be set to "active", "highlighted", etc.
     */
    selectedState: 'selected';
    /**
     * The event name to trigger when select/unselect an item.
     */
    eventName: string;
    /**
     * The shape style of the brush while selecting.
     */
    brushStyle: {
        fill?: string;
        stroke?: string;
        fillOpacity?: number;
        lineWidth?: number;
        [key: string]: unknown;
    };
    /** The mode to compose the selections from times of brush */
    selectSetMode: 'union' | 'intersect' | 'diff' | 'latest';
    /**
     * Whether allow the behavior happen on the current item.
     */
    shouldBegin: (event: IG6GraphEvent) => boolean;
    /**
     * Whether to update item state.
     * If it returns false, you may probably listen to `eventName` and
     * manage states or data manually
     */
    shouldUpdate: (itemType: ITEM_TYPE, id: ID, action: 'select' | 'deselect', self: BrushSelect) => boolean;
    /**
     * A callback be called after selecting.
     */
    onSelect: (selectedIds: {
        nodes: ID[];
        edges: ID[];
        combos: ID[];
    }) => void;
    /**
     * A callback be called after deselecting.
     */
    onDeselect: (selectedIds: {
        nodes: ID[];
        edges: ID[];
        combos: ID[];
    }, deselectedIds: {
        nodes: ID[];
        edges: ID[];
        combos: ID[];
    }) => void;
}
export declare class BrushSelect extends Behavior {
    brush: DisplayObject | undefined;
    selectedIds: IDSet | undefined;
    beginPoint: Point;
    dragging: boolean;
    mousedown: boolean;
    constructor(options: Partial<BrushSelectOptions>);
    getEvents: () => {
        pointerdown: (event: IG6GraphEvent) => void;
        pointermove: (event: IG6GraphEvent) => void;
        pointerup: (event: IG6GraphEvent) => void;
    };
    private isKeydown;
    onMouseDown(event: IG6GraphEvent): void;
    onMouseMove(event: IG6GraphEvent): void;
    onMouseUp(event: IG6GraphEvent): void;
    clearStates(clearIds?: IDSet | undefined, restIds?: any): void;
    selectItems(event: IG6GraphEvent): IDSet;
    createBrush(): DisplayObject<any, any>;
    updateBrush(event: IG6GraphEvent): DisplayObject<any, any>;
    removeBrush(): void;
    getSelector(): (graph: import("../..").IGraph<import("../../types/behavior").BehaviorRegistry, import("../../types/theme").ThemeRegistry>, points: Point[], itemTypes: ITEM_TYPE[]) => {
        nodes: ID[];
        edges: ID[];
        combos: ID[];
    };
    getPoints(event: IG6GraphEvent): (Point | import("@antv/g").Point)[];
    destroy(): void;
}
export {};
