import { Behavior } from '../../types/behavior';
import { IG6GraphEvent } from '../../types/event';
declare const ALLOWED_TRIGGERS: readonly ["shift", "ctrl", "alt", "meta"];
type Trigger = (typeof ALLOWED_TRIGGERS)[number];
export interface ClickSelectOptions {
    /**
     * Whether to allow multiple selection.
     * Defaults to true.
     * If set to false, `trigger` options will be ignored.
     */
    multiple: boolean;
    /**
     * The key to pressed with mouse click to apply multiple selection.
     * Defaults to `"shift"`.
     * Could be "shift", "ctrl", "alt", or "meta".
     */
    trigger: Trigger;
    /**
     * Item types to be able to select.
     * Defaults to `["node"]`.
     * Should be an array of "node", "edge", or "combo".
     */
    itemTypes: Array<'node' | 'edge' | 'combo'>;
    /**
     * The state to be applied when select.
     * Defaults to `"selected"`.
     * Can be set to "active", "highlighted", etc.
     */
    selectedState: 'selected';
    /**
     * The event name to trigger when select/unselect an item.
     */
    eventName: string;
    /**
     * Whether allow the behavior happen on the current item.
     */
    shouldBegin: (event: IG6GraphEvent) => boolean;
    /**
     * Whether to update item state.
     * If it returns false, you may probably listen to `eventName` and
     * manage states or data manually
     */
    shouldUpdate: (event: IG6GraphEvent) => boolean;
}
export declare class ClickSelect extends Behavior {
    /**
     * Cache the ids of items selected by this behavior
     */
    private selectedIds;
    /**
     * Two flag to avoid onCanvasClick triggered while dragging canvas
     */
    private canvasPointerDown;
    private canvasPointerMove;
    private timeout;
    constructor(options: Partial<ClickSelectOptions>);
    getEvents: () => {
        'node:click': (event: IG6GraphEvent) => void;
        'edge:click': (event: IG6GraphEvent) => void;
        'combo:click': (event: IG6GraphEvent) => void;
        'canvas:pointerdown': (event: IG6GraphEvent) => void;
        'canvas:pointermove': (event: IG6GraphEvent) => void;
        'canvas:pointerup': (event: IG6GraphEvent) => void;
    };
    private isMultipleSelect;
    onClickBesideDblClick(event: IG6GraphEvent): void;
    onClick(event: IG6GraphEvent): void;
    onCanvasPointerDown(event: IG6GraphEvent): void;
    onCanvasPointerMove(event: IG6GraphEvent): void;
    onCanvasPointerUp(event: IG6GraphEvent): void;
    onCanvasClick(event: IG6GraphEvent): void;
}
export {};
