import { IG6GraphEvent } from 'types';
import { Behavior } from '../../types/behavior';
export interface DragCanvasOptions {
    /**
     * Whether enable optimize strategies, which will hide all the shapes excluding node keyShape while dragging.
     */
    enableOptimize?: boolean;
    /**
     * Whether allow trigger this behavior when drag start on nodes / edges / combos.
     */
    dragOnItems?: boolean;
    /**
     * The trigger for the behavior, 'drag' by default. 'directionKeys' means trigger this behavior by up / down / left / right keys on keyboard.
     */
    trigger?: 'drag' | 'directionKeys';
    /**
     * The direction to drag the canvas. 'both' by default.
     */
    direction?: 'both' | 'x' | 'y';
    /**
     * The assistant secondary key on keyboard. If it is not assigned, the behavior will be triggered when trigger happens.
     */
    secondaryKey?: string;
    /**
     * The assistant secondary key on keyboard to prevent the behavior to be tiggered. 'shift' by default.
     */
    secondaryKeyToDisable?: string;
    /**
     * The key on keyboard to speed up translating while pressing and drag-canvas by direction keys. The trigger should be 'directionKeys' for this option.
     */
    speedUpKey?: string;
    /**
     * The range of canvas to limit dragging, 0 by default, which means the graph cannot be dragged totally out of the view port range.
     * If scalableRange is number or a string without 'px', means it is a ratio of the graph content.
     * If scalableRange is a string with 'px', it is regarded as pixels.
     * If scalableRange = 0, no constrains;
     * If scalableRange > 0, the graph can be dragged out of the view port range
     * If scalableRange < 0, the range is smaller than the view port.
     * Refer to https://gw.alipayobjects.com/mdn/rms_f8c6a0/afts/img/A*IFfoS67_HssAAAAAAAAAAAAAARQnAQ
     */
    scalableRange?: string | number;
    /**
     * The event name to trigger when drag end.
     */
    eventName?: string;
    /**
     * Whether allow the behavior happen on the current item.
     */
    shouldBegin?: (event: IG6GraphEvent) => boolean;
}
export declare class DragCanvas extends Behavior {
    private pointerDownAt;
    private dragging;
    private keydown;
    private speedupKeydown;
    private disableKeydown;
    private hiddenEdgeIds;
    private hiddenNodeIds;
    constructor(options: Partial<DragCanvasOptions>);
    getEvents(): {
        keydown: (event: any) => void;
        keyup: (event: any) => void;
        pointerdown?: undefined;
        pointermove?: undefined;
        pointerup?: undefined;
    } | {
        pointerdown: (event: any) => void;
        pointermove: (event: any) => void;
        pointerup: () => void;
        keydown: (event: any) => void;
        keyup: (event: any) => void;
    };
    onPointerDown(event: any): void;
    private hideShapes;
    private formatDisplacement;
    onPointerMove(event: any): void;
    onPointerUp(): void;
    onKeydown(event: any): void;
    onKeyup(event: any): void;
    destroy(): void;
}
