import { ID } from '@antv/graphlib';
import { Behavior } from '../../types/behavior';
import { IG6GraphEvent } from '../../types/event';
export interface DragComboOptions {
    /**
     * Whether to draw dragging nodes in transient layer.
     * Ignored when enableDelegate is true.
     * Defaults to true.
     */
    enableTransient?: boolean;
    /**
     * Whether to use a virtual rect moved with the dragging mouse instead of the node.
     * Defaults to false.
     */
    enableDelegate?: boolean;
    /**
     * The drawing properties when the nodes are dragged.
     * Only used when enableDelegate is true.
     */
    delegateStyle?: {
        fill?: string;
        stroke?: string;
        fillOpacity?: number;
        strokeOpacity?: number;
        lineWidth?: number;
        lineDash?: [number, number];
        [key: string]: unknown;
    };
    /**
     * The time in milliseconds to throttle moving. Useful to avoid the frequent calculation.
     * Defaults to 0.
     */
    throttle?: number;
    /**
     * Whether to hide the related edges to avoid calculation while dragging nodes.
     * Ignored when enableTransient or enableDelegate is true.
     * Defaults to false.
     */
    hideRelatedEdges?: boolean;
    /**
     * The state name to be considered as "selected".
     * Defaults to "selected".
     */
    selectedState?: string;
    /**
     * The event name to trigger when drag end.
     */
    eventName?: string;
    /**
     * Whether change the combo hierarchy structure or only change size.
     */
    updateComboStructure?: boolean;
    /**
     * Whether allow the behavior happen on the current item.
     */
    shouldBegin?: (event: IG6GraphEvent) => boolean;
}
type Position = {
    id: ID;
    x: number;
    y: number;
    minX?: number;
    maxX?: number;
    minY?: number;
    maxY?: number;
};
export declare class DragCombo extends Behavior {
    private hiddenEdges;
    private hiddenComboTreeRoots;
    private originX;
    private originY;
    private previousX;
    private previousY;
    private originPositions;
    private selectedComboIds;
    private pointerDown;
    private dragging;
    constructor(options: Partial<DragComboOptions>);
    getEvents: () => {
        'combo:pointerdown': (event: IG6GraphEvent) => void;
        pointermove: (event: IG6GraphEvent) => void;
        click: (event: IG6GraphEvent) => void;
        keydown: any;
        'node:drop': (event: IG6GraphEvent) => void;
        'combo:drop': (event: IG6GraphEvent) => void;
        'canvas:drop': (event: IG6GraphEvent) => void;
    } | {
        'combo:pointerdown': (event: IG6GraphEvent) => void;
        pointermove: (event: IG6GraphEvent) => void;
        click: (event: IG6GraphEvent) => void;
        keydown: any;
        pointerup: (event: IG6GraphEvent) => void;
    };
    /** Given selected node ids, get their related visible edges. */
    private getRelatedEdges;
    private getComboTreeItems;
    onPointerDown(event: IG6GraphEvent): void;
    onPointerMove(event: IG6GraphEvent): void;
    moveCombos(deltaX: number, deltaY: number, transient: boolean, upsertAncestors?: boolean, callback?: (positions: Position[]) => void): void;
    throttledMoveCombos: Function;
    moveDelegate(deltaX: number, deltaY: number): void;
    clearDelegate(): void;
    clearTransientItems(): void;
    restoreHiddenItems(positions?: Position[]): void;
    onPointerUp(event: IG6GraphEvent): void;
    onKeydown(event: KeyboardEvent): void;
    onDropNode(event: IG6GraphEvent): void;
    onDropCombo(event: IG6GraphEvent): void;
    onDropCanvas(event: IG6GraphEvent): void;
}
export {};
