"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HoverActivate = void 0;
const behavior_1 = require("../../types/behavior");
const DEFAULT_OPTIONS = {
    throttle: 16,
    activateState: 'active',
    eventName: '',
    itemTypes: ['node', 'edge', 'combo'],
    shouldBegin: () => true,
};
class HoverActivate extends behavior_1.Behavior {
    constructor(options) {
        const finalOptions = Object.assign({}, DEFAULT_OPTIONS, options);
        super(finalOptions);
        this.getEvents = () => {
            return {
                'node:pointerenter': this.onPointerEnter,
                'node:pointerleave': this.onPointerLeave,
                'edge:pointerenter': this.onPointerEnter,
                'edge:pointerleave': this.onPointerLeave,
                'combo:pointerenter': this.onPointerEnter,
                'combo:pointerleave': this.onPointerLeave,
            };
        };
    }
    onPointerEnter(event) {
        const { itemId, itemType } = event;
        const { graph, currentItemInfo } = this;
        const { activateState, itemTypes, eventName } = this.options;
        if (currentItemInfo && itemTypes.includes(currentItemInfo.itemType)) {
            graph.setItemState(currentItemInfo.id, activateState, false);
        }
        if (!itemTypes.includes(itemType))
            return;
        graph.setItemState(itemId, activateState, true);
        this.currentItemInfo = { id: itemId, itemType: itemType };
        // Emit event.
        if (eventName) {
            this.graph.emit(eventName, {
                itemId,
                itemType,
                state: activateState,
                action: 'pointerenter',
            });
        }
    }
    onPointerLeave(event) {
        const { itemId, itemType } = event;
        const { activateState, itemTypes, eventName } = this.options;
        if (!itemTypes.includes(itemType))
            return;
        const { graph } = this;
        graph.setItemState(itemId, activateState, false);
        this.currentItemInfo = undefined;
        // Emit event.
        if (eventName) {
            this.graph.emit(eventName, {
                itemId,
                itemType,
                state: activateState,
                action: 'pointerleave',
            });
        }
    }
}
exports.HoverActivate = HoverActivate;
//# sourceMappingURL=hover-activate.js.map