"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MapNodeSize = void 0;
/**
 * Validate and format the graph data.
 * @param data input user data.
 * @param userGraphCore the graph core stores the previous data.
 * @returns formatted data.
 */
const MapNodeSize = (data, options = {}, userGraphCore) => {
    const { field, range = [8, 40] } = options;
    if (!field)
        return data;
    const { nodes } = data;
    const nodeMap = new Map();
    // map the value to node size
    let maxValue = -Infinity, minValue = Infinity;
    nodes.forEach((n) => {
        nodeMap.set(n.id, n);
        if (maxValue < n.data[field])
            maxValue = n.data[field];
        if (minValue > n.data[field])
            minValue = n.data[field];
    });
    const valueRange = [minValue, maxValue];
    const sizeMap = scaleNodeProp(nodes, field, valueRange, range);
    sizeMap.forEach((val, id) => {
        let value = val;
        if (isNaN(val))
            value = range[0];
        const node = nodeMap.get(id);
        node.data.keyShape = Object.assign(Object.assign({}, node.data.keyShape), { r: value / 2, width: value, height: value });
    });
    return data;
};
exports.MapNodeSize = MapNodeSize;
const scaleNodeProp = (nodes, field, valueRange, mappedRange) => {
    const outLength = mappedRange[1] - mappedRange[0];
    const dataLength = valueRange[1] - valueRange[0];
    const map = new Map();
    nodes.forEach((n) => {
        map.set(n.id, ((n.data[field] - valueRange[0]) * outLength) / dataLength +
            mappedRange[0]);
    });
    return map;
};
//# sourceMappingURL=mapNodeSize.js.map