"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CubicHorizontalEdge = void 0;
const cubic_1 = require("./cubic");
class CubicHorizontalEdge extends cubic_1.CubicEdge {
    constructor(props) {
        super(props);
        this.type = 'cubic-horizontal-edge';
        this.defaultStyles = {
            keyShape: {
                x1: 0,
                y1: 0,
                z1: 0,
                x2: 0,
                y2: 0,
                z2: 0,
                isBillboard: true,
            },
        };
        /**
         * calculate the control points by curvePosition|controlPoints|curveOffset
         * @param startPoint: source point position of edge
         * @param endPoint target point position of edge
         * @param percent the proportion of control points' in the segment, Range 0 to 1
         * @param controlPoints the control point position
         * @param offset the curveOffset
         * @returns control points
         */
        this.getControlPoints = (startPoint, endPoint, percent = 0.5, controlPoints, offset) => {
            if ((controlPoints === null || controlPoints === void 0 ? void 0 : controlPoints.length) > 1)
                return controlPoints;
            const distanceToPoint = (startPoint.x - endPoint.x) * percent;
            const controlPoint1 = this.getControlPoint(startPoint, endPoint, distanceToPoint);
            const controlPoint2 = this.getControlPoint(endPoint, startPoint, -distanceToPoint);
            return [controlPoint1, controlPoint2];
        };
        /**
         * control point calculated according to startPoint, endPoint, percent, and offset
         * @param  {Point} startPoint source point position of edge (x, y)
         * @param  {Point} endPoint  target point position of edge (x, y)
         * @param  {Number} percent   the proportion of control points' in the segment, Range 0 to 1
         * @param  {Number} offset    the curveOffset
         * @return {Point} control point (x,y)
         */
        this.getControlPoint = (startPoint, endPoint, offset = 0) => {
            if (endPoint.y === startPoint.y || endPoint.x === startPoint.x) {
                return { x: (startPoint.x + endPoint.x) / 2, y: startPoint.y };
            }
            const point = {
                x: startPoint.x - offset,
                y: startPoint.y,
            };
            return point;
        };
        // suggest to merge default styles like this to avoid style value missing
        // this.defaultStyles = mergeStyles([this.baseDefaultStyles, this.defaultStyles]);
    }
}
exports.CubicHorizontalEdge = CubicHorizontalEdge;
//# sourceMappingURL=cubic-horizontal.js.map