"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PolylineEdge = void 0;
const util_1 = require("@antv/util");
const polyline_1 = require("../../../util/polyline");
const router_1 = require("../../../util/router");
const line_1 = require("./line");
class PolylineEdge extends line_1.LineEdge {
    constructor(props) {
        super(props);
        this.type = 'polyline-edge';
        this.defaultStyles = {
            keyShape: {
                x1: 0,
                y1: 0,
                z1: 0,
                x2: 0,
                y2: 0,
                z2: 0,
                isBillboard: true,
            },
        };
        // suggest to merge default styles like this to avoid style value missing
        // this.defaultStyles = mergeStyles([this.baseDefaultStyles, this.defaultStyles]);
    }
    draw(model, sourcePoint, targetPoint, shapeMap, diffData, diffState) {
        const { data = {} } = model;
        const shapes = { keyShape: undefined };
        shapes.keyShape = this.drawKeyShape(model, sourcePoint, targetPoint, shapeMap, diffData);
        if (data.haloShape) {
            shapes.haloShape = this.drawHaloShape(model, shapeMap, diffData);
        }
        if (data.labelShape) {
            shapes.labelShape = this.drawLabelShape(model, shapeMap, diffData);
        }
        // labelBackgroundShape
        if (data.labelBackgroundShape) {
            shapes.labelBackgroundShape = this.drawLabelBackgroundShape(model, shapeMap, diffData);
        }
        if (data.iconShape) {
            shapes.iconShape = this.drawIconShape(model, shapeMap, diffData);
        }
        // TODO: other shapes
        return shapes;
    }
    getControlPoints(model, sourcePoint, targetPoint) {
        const { keyShape: keyShapeStyle } = this.mergedStyles;
        return keyShapeStyle.controlPoints;
    }
    /**
     * Get polyline path
     * @param model edge display model
     * @param points lists of given 2d points
     * @param radius radius of corner
     * @param routeCfg router config
     * @param auto whether calculate the path with A*
     * @returns
     */
    getPath(model, points, radius, routeCfg, auto) {
        const { id: edgeId, source: sourceNodeId, target: targetNodeId } = model;
        // Draw a polyline with control points
        if (!auto)
            return (0, polyline_1.getPolylinePath)(edgeId, points, radius);
        // Find the shortest path computed by A* routing algorithm
        const polylinePoints = (0, router_1.pathFinder)(sourceNodeId, targetNodeId, this.nodeMap, routeCfg);
        return (0, polyline_1.getPolylinePath)(edgeId, polylinePoints, radius);
    }
    drawKeyShape(model, sourcePoint, targetPoint, shapeMap, diffData, diffState) {
        const { keyShape: keyShapeStyle } = this.mergedStyles;
        const controlPoints = this.getControlPoints(model, sourcePoint, targetPoint);
        let points = [sourcePoint, targetPoint];
        if (controlPoints) {
            points = [sourcePoint, ...controlPoints, targetPoint];
        }
        const routeCfg = (0, util_1.mix)({}, { offset: keyShapeStyle.offset }, keyShapeStyle.routeCfg);
        const path = this.getPath(model, points, keyShapeStyle.radius, routeCfg, (0, util_1.isEmpty)(controlPoints));
        const { startArrow, endArrow } = keyShapeStyle, others = __rest(keyShapeStyle, ["startArrow", "endArrow"]);
        const lineStyle = Object.assign(Object.assign({}, others), { path, isBillboard: true });
        this.upsertArrow('start', startArrow, others, model, lineStyle);
        this.upsertArrow('end', endArrow, others, model, lineStyle);
        return this.upsertShape('path', 'keyShape', lineStyle, shapeMap, model);
    }
}
exports.PolylineEdge = PolylineEdge;
//# sourceMappingURL=polyline.js.map