import { DisplayObject } from '@antv/g';
import { NodeDisplayModel } from '../../../types';
import { SHAPE_TYPE, SHAPE_TYPE_3D, ShapeStyle, State } from '../../../types/item';
import { NodeModelData, NodeShapeMap, NodeShapeStyles } from '../../../types/node';
import { BaseNode } from './base';
export declare abstract class BaseNode3D extends BaseNode {
    type: string;
    defaultStyles: NodeShapeStyles;
    themeStyles: NodeShapeStyles;
    mergedStyles: NodeShapeStyles;
    device: any;
    constructor(props: any);
    /**
     * Draw the label shape of the 3D node
     * @param model The displayed model of this 3D node, only for drawing and not received by users.
     * @param shapeMap The shape map that contains all of the elements to show on the 3D node.
     * @param diffData An object that contains previous and current data.
     * @param diffState An object that contains previous and current 3D node's state.
     * @returns The display object representing the label shape of the 3D node.
     */
    drawLabelShape(model: NodeDisplayModel, shapeMap: NodeShapeMap, diffData?: {
        previous: NodeModelData;
        current: NodeModelData;
    }, diffState?: {
        previous: State[];
        current: State[];
    }): DisplayObject;
    /**
     * Draw the icon shape of the 3D node
     * @param model The displayed model of this 3D node, only for drawing and not received by users.
     * @param shapeMap The shape map that contains all of the elements to show on the 3D node.
     * @param diffData An object that contains previous and current data.
     * @param diffState An object that contains previous and current 3D node's state.
     * @returns The display object representing the icon shape of the 3D node.
     */
    drawIconShape(model: NodeDisplayModel, shapeMap: NodeShapeMap, diffData?: {
        previous: NodeModelData;
        current: NodeModelData;
    }, diffState?: {
        previous: State[];
        current: State[];
    }): DisplayObject;
    drawHaloShape(model: NodeDisplayModel, shapeMap: NodeShapeMap, diffData?: {
        previous: NodeModelData;
        current: NodeModelData;
    }, diffState?: {
        previous: State[];
        current: State[];
    }): DisplayObject;
    /**
     * 3D node does not support anchor shapes.
     * @param model
     * @param shapeMap
     * @param diffData
     * @param diffState
     * @returns
     */
    drawAnchorShapes(model: NodeDisplayModel, shapeMap: NodeShapeMap, diffData?: {
        previous: NodeModelData;
        current: NodeModelData;
    }, diffState?: {
        previous: State[];
        current: State[];
    }): {
        [shapeId: string]: DisplayObject;
    };
    drawBadgeShapes(model: NodeDisplayModel, shapeMap: NodeShapeMap, diffData?: {
        previous: NodeModelData;
        current: NodeModelData;
    }, diffState?: {
        previous: State[];
        current: State[];
    }): {
        [shapeId: string]: DisplayObject;
    };
    drawOtherShapes(model: NodeDisplayModel, shapeMap: NodeShapeMap, diffData?: {
        previous: NodeModelData;
        current: NodeModelData;
    }, diffState?: {
        previous: State[];
        current: State[];
    }): {
        [id: string]: DisplayObject;
    };
    upsertShape(type: SHAPE_TYPE_3D | SHAPE_TYPE, id: string, style: ShapeStyle, shapeMap: {
        [shapeId: string]: DisplayObject;
    }): DisplayObject;
    /**
     * The listener for graph zooming.
     * 1. show / hide some shapes while zoom level changed;
     * 2. change the shapes' sizes to make them have same visual size while zooming, e.g. labelShape, labelBackgroundShape.
     * @param shapeMap The shape map that contains all of the elements to show on the node.
     * @param zoom The zoom level of the graph.
     */
    onZoom: (shapeMap: NodeShapeMap, zoom: number) => void;
}
