"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiamondNode = void 0;
const base_1 = require("./base");
class DiamondNode extends base_1.BaseNode {
    constructor(props) {
        super(props);
        this.defaultStyles = {
            keyShape: {
                size: [50, 30],
                x: 0,
                y: 0,
            },
        };
        // suggest to merge default styles like this to avoid style value missing
        // this.defaultStyles = mergeStyles([this.baseDefaultStyles, this.defaultStyles]);
    }
    draw(model, shapeMap, diffData, diffState) {
        const { data = {} } = model;
        let shapes = { keyShape: undefined };
        // keyShape
        shapes.keyShape = this.drawKeyShape(model, shapeMap, diffData);
        // haloShape
        if (data.haloShape && this.drawHaloShape) {
            shapes.haloShape = this.drawHaloShape(model, shapeMap, diffData);
        }
        // labelShape
        if (data.labelShape) {
            shapes.labelShape = this.drawLabelShape(model, shapeMap, diffData);
        }
        // labelBackgroundShape
        if (data.labelBackgroundShape) {
            shapes.labelBackgroundShape = this.drawLabelBackgroundShape(model, shapeMap, diffData);
        }
        // anchor shapes
        if (data.anchorShapes) {
            const anchorShapes = this.drawAnchorShapes(model, shapeMap, diffData, diffState);
            shapes = Object.assign(Object.assign({}, shapes), anchorShapes);
        }
        // iconShape
        if (data.iconShape) {
            shapes.iconShape = this.drawIconShape(model, shapeMap, diffData);
        }
        // badgeShape
        if (data.badgeShapes) {
            const badgeShapes = this.drawBadgeShapes(model, shapeMap, diffData, diffState);
            shapes = Object.assign(Object.assign({}, shapes), badgeShapes);
        }
        // otherShapes
        if (data.otherShapes && this.drawOtherShapes) {
            shapes = Object.assign(Object.assign({}, shapes), this.drawOtherShapes(model, shapeMap, diffData));
        }
        return shapes;
    }
    drawKeyShape(model, shapeMap, diffData, diffState) {
        const { keyShape: keyShapeStyle } = this.mergedStyles;
        return this.upsertShape('path', 'keyShape', Object.assign(Object.assign({}, this.mergedStyles.keyShape), { path: this.getDiamondPath(keyShapeStyle) }), shapeMap, model);
    }
    getDiamondPath(keyShapeStyle) {
        const width = keyShapeStyle.size[0];
        const height = keyShapeStyle.size[1];
        const path = [
            ['M', 0, -height / 2],
            ['L', width / 2, 0],
            ['L', 0, height / 2],
            ['L', -width / 2, 0],
            ['Z'], // close
        ];
        return path;
    }
    calculateAnchorPosition(keyShapeStyle) {
        const anchorPositionMap = {};
        const width = keyShapeStyle.size[0];
        const height = keyShapeStyle.size[1];
        anchorPositionMap['bottom'] = [0, height / 2];
        anchorPositionMap['left'] = [-width / 2, 0];
        anchorPositionMap['right'] = anchorPositionMap['default'] = [width / 2, 0];
        anchorPositionMap['top'] = [0, -height / 2];
        return anchorPositionMap;
    }
}
exports.DiamondNode = DiamondNode;
//# sourceMappingURL=diamond.js.map