"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelRectNode = void 0;
const type_1 = require("../../../util/type");
const base_1 = require("./base");
class ModelRectNode extends base_1.BaseNode {
    constructor(props) {
        super(props);
        this.defaultStyles = {
            keyShape: {
                x: 0,
                y: 0,
                width: 185,
                height: 70,
                stroke: '#C0DCEA',
                strokeWidth: '2',
                radius: 2,
                fill: 'white',
            },
        };
        // suggest to merge default styles like this to avoid style value missing
        // this.defaultStyles = mergeStyles([this.baseDefaultStyles, this.defaultStyles]);
    }
    draw(model, shapeMap, diffData, diffState) {
        const { data = {} } = model;
        let shapes = { keyShape: undefined };
        // keyShape
        shapes.keyShape = this.drawKeyShape(model, shapeMap, diffData);
        // haloShape
        if (data.haloShape && this.drawHaloShape) {
            shapes.haloShape = this.drawHaloShape(model, shapeMap, diffData);
        }
        // anchor shapes
        if (data.anchorShapes) {
            const anchorShapes = this.drawAnchorShapes(model, shapeMap, diffData, diffState);
            shapes = Object.assign(Object.assign({}, shapes), anchorShapes);
        }
        //logoIconShape
        if (data.iconShape) {
            shapes.iconShape = this.drawIconShape(model, shapeMap, diffData);
        }
        // otherShapes
        if (data.otherShapes && this.drawOtherShapes) {
            shapes = Object.assign(Object.assign({}, shapes), this.drawOtherShapes(model, shapeMap, diffData));
        }
        // labelShape (after drawOtherShapes)
        if (data.labelShape) {
            shapes.labelShape = this.drawLabelShape(model, shapeMap, diffData);
        }
        return shapes;
    }
    drawKeyShape(model, shapeMap, diffData, diffState) {
        const { x, y, height, width } = this.mergedStyles.keyShape;
        return this.upsertShape('rect', 'keyShape', Object.assign(Object.assign({}, this.mergedStyles.keyShape), { x: (0, type_1.convertToNumber)(x) - (0, type_1.convertToNumber)(width) / 2, y: (0, type_1.convertToNumber)(y) - (0, type_1.convertToNumber)(height) / 2 }), shapeMap, model);
    }
    calculateAnchorPosition(keyShapeStyle) {
        const x = (0, type_1.convertToNumber)(keyShapeStyle.x);
        const y = (0, type_1.convertToNumber)(keyShapeStyle.y);
        const height = keyShapeStyle.height;
        const width = keyShapeStyle.width;
        const leftX = x - width / 2 - (this.mergedStyles.preRect ? 5 : 0);
        const anchorPositionMap = {};
        anchorPositionMap['top'] = [x, y - height / 2];
        anchorPositionMap['left'] = [leftX, y];
        anchorPositionMap['right'] = anchorPositionMap['default'] = [
            x + width / 2,
            y,
        ];
        anchorPositionMap['bottom'] = [x, y + height / 2];
        return anchorPositionMap;
    }
    drawLabelShape(model, shapeMap, diffData, diffState) {
        const { labelShape: labelShapeStyle, description: descriptionStyle, keyShape: keyShapeStyle, } = this.mergedStyles;
        if (!labelShapeStyle)
            return;
        const { width, height, x, y } = keyShapeStyle;
        const { offsetX: labelOffsetX = 0, offsetY: labelOffsetY = 0 } = labelShapeStyle;
        const mixDisplay = labelShapeStyle && descriptionStyle ? true : false; //false: only display label
        const defaultLabelFontSize = mixDisplay
            ? Math.min(height, width) / 5
            : Math.min(height, width) / 6; //match with keyShape
        const defaultLabelX = mixDisplay
            ? this.logoX + this.logoWidth + width / 20
            : 0;
        const defaultLabelY = mixDisplay ? 0 - height / 7 : 0;
        const defaultWordWrapWidth = width / 2 -
            (this.logoX + this.logoWidth + width / 20) -
            defaultLabelFontSize * 2;
        labelShapeStyle.maxWidth = defaultWordWrapWidth;
        return this.upsertShape('text', 'labelShape', Object.assign({ wordWrap: true, textOverflow: 'ellipsis', textBaseline: 'middle', textAlign: mixDisplay ? 'left' : 'middle', x: defaultLabelX + labelOffsetX, y: defaultLabelY + labelOffsetY, maxLines: 1, wordWrapWidth: defaultWordWrapWidth }, labelShapeStyle), shapeMap, model);
    }
    drawOtherShapes(model, shapeMap, diffData, diffState) {
        const { logoIcon: logoIconStyle, preRect: preRectStyle, stateIcon: stateIconStyle, description: descriptionStyle, label: labelStyle, keyShape: keyShapeStyle, } = this.mergedStyles;
        return Object.assign(Object.assign(Object.assign(Object.assign({}, this.drawPreRectShape(preRectStyle, keyShapeStyle, model, shapeMap)), this.drawLogoIconShape(logoIconStyle, keyShapeStyle, model, shapeMap)), this.drawDescriptionShape(descriptionStyle, keyShapeStyle, model, shapeMap)), this.drawStateIcon(stateIconStyle, keyShapeStyle, model, shapeMap));
    }
    drawPreRectShape(preRectStyle, keyShapeStyle, model, shapeMap) {
        if (!preRectStyle)
            return;
        const { x, y, height, width } = keyShapeStyle;
        const { width: preRectWidth = 5, fill = '#5CAAF8' } = preRectStyle;
        return {
            otherShapePreRect: this.upsertShape('rect', 'otherShapePreRect', {
                width: preRectWidth,
                height,
                x: (0, type_1.convertToNumber)(x) - (0, type_1.convertToNumber)(width) / 2 - preRectWidth,
                y: (0, type_1.convertToNumber)(y) - (0, type_1.convertToNumber)(height) / 2,
                fill,
                radius: 2,
            }, shapeMap, model),
        };
    }
    drawLogoIconShape(logoIconStyle, keyShapeStyle, model, shapeMap) {
        const { x, y, width, height } = keyShapeStyle;
        if (!logoIconStyle) {
            this.logoWidth = 16;
            this.logoHeight = 16;
            this.logoX =
                (0, type_1.convertToNumber)(x) -
                    (0, type_1.convertToNumber)(width) / 2 +
                    width / 10 -
                    this.logoWidth / 2;
            this.logoY = (0, type_1.convertToNumber)(y) - (0, type_1.convertToNumber)(this.logoHeight) / 2;
            return;
        }
        const { width: logoIconWidth = 16, height: logoIconHeight = 16, fontSize: logoIconFontSize, text: logoIconText, offsetX: logoIconOffsetX = 0, offsetY: logoIconOffsetY = 0, } = logoIconStyle;
        const logoWidth = (this.logoWidth = (0, type_1.convertToNumber)(logoIconWidth || logoIconFontSize));
        const logoHeight = (this.logoHeight = (0, type_1.convertToNumber)(logoIconWidth || logoIconFontSize));
        const logoIconShapeType = logoIconText ? 'text' : 'image';
        //calculate logo position
        const logoX = (this.logoX =
            (0, type_1.convertToNumber)(x) -
                (0, type_1.convertToNumber)(width) / 2 +
                logoIconOffsetX +
                width / 10 -
                logoIconWidth / 2);
        const logoY = (this.logoY = logoIconText
            ? logoIconOffsetY
            : (0, type_1.convertToNumber)(y) -
                (0, type_1.convertToNumber)(logoIconHeight) / 2 +
                logoIconOffsetY);
        if (logoIconText) {
            logoIconStyle.textAlign = 'center';
            logoIconStyle.textBaseline = 'middle';
        }
        const shapes = {};
        shapes['otherShapeLogoIcon'] = this.upsertShape(logoIconShapeType, 'otherShapeLogoIcon', Object.assign({ width: logoWidth, height: logoHeight, x: logoX, y: logoY }, logoIconStyle), shapeMap, model);
        return shapes;
    }
    drawDescriptionShape(descriptionStyle, keyShapeStyle, model, shapeMap) {
        if (!descriptionStyle)
            return;
        const { width, height } = keyShapeStyle;
        const { text: descriptionText, fontSize, offsetX: descriptionOffsetX = 0, offsetY: descriptionOffsetY = 0, } = descriptionStyle;
        const defaultLabelFontSize = Math.min(height, width) / 5; //match with keyShape
        const defaultDescriptionX = this.logoX + this.logoWidth + width / 20;
        const defaultWordWrapWidth = width / 2 - defaultDescriptionX - defaultLabelFontSize * 2;
        const defaultDescriptionFontSize = defaultLabelFontSize / 2;
        return {
            otherShapeDescription: this.upsertShape('text', 'otherShapeDescription', Object.assign({ textBaseline: 'middle', textAlign: 'left', fontSize: defaultDescriptionFontSize, x: defaultDescriptionX + descriptionOffsetX, y: 0 + height / 6 + descriptionOffsetY, wordWrap: true, wordWrapWidth: defaultWordWrapWidth, maxLines: 1, textOverflow: 'ellipsis', fill: '#C2C2C2' }, descriptionStyle), shapeMap, model),
        };
    }
    drawStateIcon(stateIconStyle, keyShapeStyle, model, shapeMap) {
        if (!stateIconStyle)
            return;
        const { x, y, width } = keyShapeStyle;
        const { width: stateIconWidth = 16, height: stateIconHeight = 16, fontSize: stateIconFontSize, text: stateIconText, offsetX: stateIconOffsetX = 0, offsetY: stateIconOffsetY = 0, } = stateIconStyle;
        const stateWidth = (0, type_1.convertToNumber)(stateIconWidth || stateIconFontSize);
        const stateHeight = (0, type_1.convertToNumber)(stateIconWidth || stateIconFontSize);
        const stateIconShapeType = stateIconText ? 'text' : 'image';
        //calculate state position
        const stateX = (0, type_1.convertToNumber)(x) +
            (0, type_1.convertToNumber)(width) / 2 -
            width / 10 +
            stateIconOffsetX;
        const stateY = stateIconText
            ? stateIconOffsetY
            : (0, type_1.convertToNumber)(y) -
                (0, type_1.convertToNumber)(stateIconHeight || stateIconFontSize) / 2 +
                stateIconOffsetY;
        if (stateIconText) {
            stateIconStyle.textAlign = 'center';
            stateIconStyle.textBaseline = 'middle';
        }
        return {
            otherShapestateIcon: this.upsertShape(stateIconShapeType, 'otherShapestateIcon', Object.assign({ width: stateWidth, height: stateHeight, x: stateX, y: stateY }, stateIconStyle), shapeMap, model),
        };
    }
}
exports.ModelRectNode = ModelRectNode;
//# sourceMappingURL=modelRect.js.map