"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SphereNode = void 0;
const base3d_1 = require("./base3d");
class SphereNode extends base3d_1.BaseNode3D {
    constructor(props) {
        super(props);
        this.defaultStyles = {
            keyShape: {
                r: 5,
                latitudeBands: 32,
                longitudeBands: 32,
                x: 0,
                y: 0,
                z: 0,
            },
        };
    }
    draw(model, shapeMap, diffData, diffState) {
        const { data = {} } = model;
        let shapes = { keyShape: undefined };
        // keyShape
        shapes.keyShape = this.drawKeyShape(model, shapeMap, diffData);
        // haloShape
        if (data.haloShape && this.drawHaloShape) {
            shapes.haloShape = this.drawHaloShape(model, shapeMap, diffData);
        }
        // labelShape
        if (data.labelShape) {
            shapes.labelShape = this.drawLabelShape(model, shapeMap, diffData);
        }
        // labelBackgroundShape
        if (data.labelBackgroundShape) {
            shapes.labelBackgroundShape = this.drawLabelBackgroundShape(model, shapeMap, diffData);
        }
        // iconShape
        if (data.iconShape) {
            shapes.iconShape = this.drawIconShape(model, shapeMap, diffData);
        }
        // badgeShape
        if (data.badgeShapes) {
            const badgeShapes = this.drawBadgeShapes(model, shapeMap, diffData, diffState);
            shapes = Object.assign(Object.assign({}, shapes), badgeShapes);
        }
        // otherShapes
        if (data.otherShapes && this.drawOtherShapes) {
            shapes = Object.assign(Object.assign({}, shapes), this.drawOtherShapes(model, shapeMap, diffData));
        }
        return shapes;
    }
    drawKeyShape(model, shapeMap, diffData, diffState) {
        return this.upsertShape('sphere', 'keyShape', this.mergedStyles.keyShape, shapeMap);
    }
}
exports.SphereNode = SphereNode;
//# sourceMappingURL=sphere.js.map