"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StarNode = void 0;
const base_1 = require("./base");
class StarNode extends base_1.BaseNode {
    constructor(props) {
        super(props);
        this.defaultOuterR = 20;
        this.defaultStyles = {
            keyShape: {
                outerR: this.defaultOuterR,
                innerR: (this.defaultOuterR * 3) / 8,
                x: 0,
                y: 0,
            },
        };
        // suggest to merge default styles like this to avoid style value missing
        // this.defaultStyles = mergeStyles([this.baseDefaultStyles, this.defaultStyles]);
    }
    draw(model, shapeMap, diffData, diffState) {
        const { data = {} } = model;
        let shapes = { keyShape: undefined };
        // keyShape
        shapes.keyShape = this.drawKeyShape(model, shapeMap, diffData);
        // haloShape
        if (data.haloShape && this.drawHaloShape) {
            shapes.haloShape = this.drawHaloShape(model, shapeMap, diffData);
        }
        // labelShape
        if (data.labelShape) {
            shapes.labelShape = this.drawLabelShape(model, shapeMap, diffData);
        }
        // labelBackgroundShape
        if (data.labelBackgroundShape) {
            shapes.labelBackgroundShape = this.drawLabelBackgroundShape(model, shapeMap, diffData);
        }
        // anchor shapes
        if (data.anchorShapes) {
            const anchorShapes = this.drawAnchorShapes(model, shapeMap, diffData, diffState);
            shapes = Object.assign(Object.assign({}, shapes), anchorShapes);
        }
        // iconShape
        if (data.iconShape) {
            shapes.iconShape = this.drawIconShape(model, shapeMap, diffData);
        }
        // badgeShape
        if (data.badgeShapes) {
            const badgeShapes = this.drawBadgeShapes(model, shapeMap, diffData, diffState);
            shapes = Object.assign(Object.assign({}, shapes), badgeShapes);
        }
        // otherShapes
        if (data.otherShapes && this.drawOtherShapes) {
            shapes = Object.assign(Object.assign({}, shapes), this.drawOtherShapes(model, shapeMap, diffData));
        }
        return shapes;
    }
    drawKeyShape(model, shapeMap, diffData, diffState) {
        const { keyShape: keyShapeStyle } = this.mergedStyles;
        const path = this.getStarPath(keyShapeStyle.outerR, keyShapeStyle.innerR);
        return this.upsertShape('path', 'keyShape', Object.assign(Object.assign({}, keyShapeStyle), { path }), shapeMap, model);
    }
    getStarPath(outerR, innerR) {
        return [
            ['M', 0, -outerR],
            [
                'L',
                innerR * Math.cos((3 * Math.PI) / 10),
                -innerR * Math.sin((3 * Math.PI) / 10),
            ],
            ['L', outerR * Math.cos(Math.PI / 10), -outerR * Math.sin(Math.PI / 10)],
            ['L', innerR * Math.cos(Math.PI / 10), innerR * Math.sin(Math.PI / 10)],
            [
                'L',
                outerR * Math.cos((3 * Math.PI) / 10),
                outerR * Math.sin((3 * Math.PI) / 10),
            ],
            ['L', 0, innerR],
            [
                'L',
                -outerR * Math.cos((3 * Math.PI) / 10),
                outerR * Math.sin((3 * Math.PI) / 10),
            ],
            ['L', -innerR * Math.cos(Math.PI / 10), innerR * Math.sin(Math.PI / 10)],
            ['L', -outerR * Math.cos(Math.PI / 10), -outerR * Math.sin(Math.PI / 10)],
            [
                'L',
                -innerR * Math.cos((3 * Math.PI) / 10),
                -innerR * Math.sin((3 * Math.PI) / 10),
            ],
            'Z',
        ];
    }
    calculateAnchorPosition(keyShapeStyle) {
        const anchorPositionMap = {};
        const outerR = keyShapeStyle.outerR;
        const innerR = keyShapeStyle.innerR;
        anchorPositionMap['top'] = [0, -outerR];
        anchorPositionMap['left'] = [
            -outerR * Math.cos(Math.PI / 10),
            -outerR * Math.sin(Math.PI / 10),
        ];
        anchorPositionMap['leftbottom'] = [
            -outerR * Math.cos((3 * Math.PI) / 10),
            outerR * Math.sin((3 * Math.PI) / 10),
        ];
        anchorPositionMap['bottom'] = [0, innerR];
        anchorPositionMap['rightbottom'] = [
            outerR * Math.cos((3 * Math.PI) / 10),
            outerR * Math.sin((3 * Math.PI) / 10),
        ];
        anchorPositionMap['right'] = anchorPositionMap['default'] = [
            outerR * Math.cos(Math.PI / 10),
            -outerR * Math.sin(Math.PI / 10),
        ];
        return anchorPositionMap;
    }
}
exports.StarNode = StarNode;
//# sourceMappingURL=star.js.map