"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("@antv/util");
const layer_updated_command_1 = require("./layer-updated-command");
const state_updated_command_1 = require("./state-updated-command");
const visibility_updated_command_1 = require("./visibility-updated-command");
const item_data_command_1 = require("./item-data-command");
const combo_command_1 = require("./combo-command");
class CommandFactory {
    static create(_a) {
        var { type: itemType, action, changes } = _a, rest = __rest(_a, ["type", "action", "changes"]);
        const commands = [];
        const groupedByType = (0, util_1.groupBy)(changes, 'type');
        for (const type in groupedByType) {
            const itemDataCommandMap = {
                NodeDataUpdated: 'node',
                EdgeDataUpdated: 'edge',
                NodeAdded: 'node',
                EdgeAdded: 'edge',
                NodeRemoved: 'node',
                EdgeRemoved: 'edge',
                TreeStructureChanged: 'combo',
            };
            if (itemDataCommandMap[type]) {
                commands.push(new item_data_command_1.ItemDataCommand(action, itemType || itemDataCommandMap[type], groupedByType[type], type === 'NodeDataUpdated' ? rest.upsertAncestors : undefined));
            }
        }
        const actionCommandMap = {
            updateState: new state_updated_command_1.StateUpdatedCommand(changes),
            updateVisibility: new visibility_updated_command_1.VisibilityUpdatedCommand(changes, rest.disableAnimate),
            front: new layer_updated_command_1.LayerUpdatedCommand('front', rest.ids),
            back: new layer_updated_command_1.LayerUpdatedCommand('back', rest.ids),
            expandCombo: new combo_command_1.ComboCommand('expandCombo', rest.ids),
            collapseCombo: new combo_command_1.ComboCommand('collapseCombo', rest.ids),
        };
        if (actionCommandMap[action]) {
            commands.push(actionCommandMap[action]);
        }
        return commands;
    }
}
exports.default = CommandFactory;
//# sourceMappingURL=command.js.map