"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ItemDataCommand = void 0;
const history_1 = require("../../../types/history");
class ItemDataCommand {
    constructor(action, type, changes, upsertAncestors) {
        this.action = action;
        this.type = type;
        this.changes = changes;
        this.upsertAncestors = upsertAncestors;
    }
    removeChangedData(graph) {
        const ids = this.changes.map((data) => { var _a; return (_a = data.value) === null || _a === void 0 ? void 0 : _a.id; }).filter(Boolean);
        graph.executeWithoutStacking(() => {
            graph.removeData(this.type, ids);
        });
    }
    addChangedData(graph) {
        const models = this.changes.map((data) => data.value).filter(Boolean);
        graph.executeWithoutStacking(() => {
            graph.addData(this.type, models);
        });
    }
    updateChangedData(graph, operationType, onlyMove = false) {
        let models;
        if (this.type === 'combo' && !onlyMove) {
            models = this.changes.map((data) => ({
                id: data.nodeId,
                data: {
                    parentId: history_1.STACK_TYPE.undo ? data.oldParentId : data.newParentId,
                },
            }));
        }
        else {
            models = this.changes.map((data) => {
                return {
                    id: data.id,
                    data: operationType === history_1.STACK_TYPE.undo ? data.oldValue : data.newValue,
                };
            });
        }
        graph.pauseStacking();
        if (onlyMove) {
            graph.updatePosition(this.type, models, this.upsertAncestors);
        }
        else {
            graph.updateData(this.type, models);
        }
        graph.resumeStacking();
    }
    undo(graph) {
        this.action === 'add' && this.removeChangedData(graph);
        this.action === 'remove' && this.addChangedData(graph);
        this.action === 'update' && this.updateChangedData(graph, history_1.STACK_TYPE.undo);
        this.action === 'updatePosition' &&
            this.updateChangedData(graph, history_1.STACK_TYPE.undo, this.action === 'updatePosition');
    }
    redo(graph) {
        this.action === 'remove' && this.removeChangedData(graph);
        this.action === 'add' && this.addChangedData(graph);
        this.action === 'update' && this.updateChangedData(graph, history_1.STACK_TYPE.redo);
        this.action === 'updatePosition' &&
            this.updateChangedData(graph, history_1.STACK_TYPE.redo, this.action === 'updatePosition');
    }
}
exports.ItemDataCommand = ItemDataCommand;
//# sourceMappingURL=item-data-command.js.map