import { vec2 } from '@antv/matrix-util';
import { ID, IGraph } from 'types';
import { AABB } from '@antv/g';
import { Bounds, Point } from '../../../types/common';
/**
 * Generate smooth closed curves.
 * @param points points of the curves.
 */
export declare const getClosedSpline: (points: Point[]) => any[];
/**
 * The incoming node is used as the vertices of the polygon to generate a polygon with rounded corners.
 * @param polyPoints vertex points of the polygon.
 * @param padding Increase the blank space between the final outline and the original polygon on the basis of the original polygon.
 */
export declare function roundedHull(polyPoints: vec2[], padding: number): string;
/**
 * The incoming nodes are used as polygon vertices to generate a smooth closed polygon
 * @param polyPoints vertex points of the polygon.
 * @param padding Increase the blank space between the final outline and the original polygon on the basis of the original polygon.
 */
export declare function paddedHull(polyPoints: vec2[], padding: number): string | {
    x: number;
    y: number;
}[];
export declare const pointLineSquareDist: (point: Point, line: LineStructure) => number;
export declare class LineStructure {
    x1: number;
    y1: number;
    x2: number;
    y2: number;
    constructor(x1: number, y1: number, x2: number, y2: number);
    getBBox(): Bounds;
}
export declare const getPointsCenter: (points: Point[]) => Point;
export declare const squareDist: (a: Point, b: Point) => number;
export declare const fractionToLine: (graph: IGraph, itemId: ID, line: LineStructure) => number;
export declare const getBBoxBoundLine: (bbox: AABB, direction: string) => any;
export declare const isPointsOverlap: (p1: any, p2: any, e?: number) => boolean;
export declare const itemIntersectByLine: (graph: IGraph, itemId: ID, line: LineStructure) => [Point[], number];
/**
 * The square of the distance from the point to the rectangle: the distance from the point inside the rectangle is 0, if the projection of the point outside falls on the side of the rectangle, it is the nearest vertical distance from the point to the side of the rectangle, otherwise it is the distance from the point to the vertex of the rectangle.
 * @param point Point
 * @param rect Rect
 */
export declare const pointRectSquareDist: (point: Point, rect: {
    width: number;
    height: number;
    x: number;
    y: number;
}) => number;
/**
 * Get the intersect point and line of a polygon (represented by points) and a line (represented by 2 points).
 * @param polygonPoints Vertex points of the polygon.
 * @param line Start and end points of the line.
 * @returns The intersect point and the intersected line of polygon.
 */
export declare const getPolygonIntersectByLine: (polygonPoints: [number, number][], line: [Point, Point]) => {
    point: Point;
    line: [Point, Point];
};
/**
 * Calculates the extended point along the line from point2 to point1.
 * @param point1 The first point in the line. And the extended point will on the point1 side.
 * @param point2 The second point in the line.
 * @param diff The distance by which to extend the line.
 * @returns The extended point on the line.
 */
export declare const getExtendPoint: (point1: any, point2: any, diff: any) => {
    x: any;
    y: any;
};
/**
 * Get the inverted value of baseline.
 * @param baseline
 * @returns
 */
export declare const getInvertBaseline: (baseline: string) => string;
