import Item from '../../../item/item';
import { IGraph } from '../../../types';
import { IG6GraphEvent } from '../../../types/event';
import { Plugin as Base, IPluginBaseConfig } from '../../../types/plugin';
/**
 * The `MenuConfig` interface contains the following properties:

- `handleMenuClick`: An optional function for handling menu click events. It takes two arguments: `target` (of type HTMLElement) and `item` (of type Item), and has no return value.
- `getContent`: An optional function for getting the content of the menu. It takes an optional argument of type `IG6GraphEvent`, and returns a value of type HTMLDivElement, string, or Promise (resolving to HTMLDivElement or string).
- `offsetX`: An optional number representing the offset of the menu in the X direction.
- `offsetY`: An optional number representing the offset of the menu in the Y direction.
- `shouldBegin`: An optional function for determining whether the menu should be displayed. It takes an optional argument of type `IG6GraphEvent`, and returns a boolean value.
- `itemTypes`: An optional array of strings representing the types of items for which the menu is allowed to be displayed.
- `trigger`: An optional string, either 'click' or 'contextmenu', representing the event type that triggers the display of the menu.
- `onHide`: An optional function to be executed when the menu is hidden. It takes no arguments and returns a boolean value.
- `loadingContent`: An optional HTMLDivElement or string representing the loading DOM.
- `liHoverStyle`: An optional object representing the style of li elements when hovered over. It can contain any number of key-value pairs, where the key is a style name and the value is a string.
 */
export interface MenuConfig extends IPluginBaseConfig {
    handleMenuClick?: (target: HTMLElement, item: Item) => void;
    getContent?: (evt?: IG6GraphEvent) => HTMLDivElement | string | Promise<HTMLDivElement | string>;
    offsetX?: number;
    offsetY?: number;
    shouldBegin?: (evt?: IG6GraphEvent) => boolean;
    itemTypes?: string[];
    trigger?: 'click' | 'contextmenu';
    onHide?: () => boolean;
    loadingContent?: HTMLDivElement | string;
    liHoverStyle?: {
        [key: string]: string;
    };
}
export declare class Menu extends Base {
    private menu;
    private handleMenuClickWrapper;
    private handler;
    private currentTarget;
    private asyncMenu;
    constructor(options?: MenuConfig);
    getDefaultCfgs(): MenuConfig;
    getEvents(): {
        click: (e: IG6GraphEvent) => Promise<void>;
        touchend: (e: IG6GraphEvent) => Promise<void>;
        contextmenu?: undefined;
    } | {
        contextmenu: (e: IG6GraphEvent) => Promise<void>;
        click?: undefined;
        touchend?: undefined;
    };
    init(graph: IGraph): void;
    protected onMenuShow(e: IG6GraphEvent): Promise<void>;
    private removeMenuEventListener;
    private onMenuHide;
    destroy(): void;
}
