"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tooltip = void 0;
// TODO: update type define.
const dom_util_1 = require("@antv/dom-util");
const util_1 = require("@antv/util");
const insert_css_1 = __importDefault(require("insert-css"));
const plugin_1 = require("../../../types/plugin");
typeof document !== 'undefined' &&
    (0, insert_css_1.default)(`
    .g6-component-tooltip {
        border: 1px solid #e2e2e2;
        border-radius: 4px;
        font-size: 12px;
        color: #545454;
        background-color: rgba(255, 255, 255, 0.9);
        padding: 10px 8px;
        box-shadow: rgb(174, 174, 174) 0px 0px 10px;
    }
    .tooltip-type {
        padding: 0;
        margin: 0;
    }
    .tooltip-id {
        color: #531dab;
    }
    .g6-loading-dom {
    border: 5px solid #e5e5e5;
    border-top: 5px solid #227EFF;
    border-radius: 50%;
    width: 25px;
    height: 25px;
    animation: turn-around 1.5s linear infinite;
    }
    @keyframes turn-around {
        0% { transform: rotate(0deg); }
        100% { transform: rotate(360deg); }
    }
`);
class Tooltip extends plugin_1.Plugin {
    constructor(options) {
        super(options);
    }
    getDefaultCfgs() {
        return {
            key: `tooltip-${(0, util_1.uniqueId)()}`,
            offsetX: 0,
            offsetY: 0,
            getContent: (e) => {
                return `
        <div>
          <h4 class='tooltip-type'>类型: ${e.itemType}</h4>
          <span class='tooltip-id'>ID: ${e.itemId}</span>
        </div>
        `;
            },
            shouldBegin: (e) => {
                return true;
            },
            itemTypes: ['node', 'edge', 'combo'],
            trigger: 'pointerenter',
            fixToNode: undefined,
            loadingContent: `<div class='g6-loading-dom'></div>`,
        };
    }
    getEvents() {
        if (this.options.trigger === 'click') {
            return {
                'node:click': this.onClick,
                'edge:click': this.onClick,
                'combo:click': this.onClick,
                'canvas:click': this.onPointerLeave,
                afterremoveitem: this.onPointerLeave,
                contextmenu: this.onPointerLeave,
                drag: this.onPointerLeave,
            };
        }
        return {
            'node:pointerenter': this.onPointerEnter,
            'node:pointerleave': this.onPointerLeave,
            'node:pointermove': this.onPointerMove,
            'edge:pointerenter': this.onPointerEnter,
            'edge:pointerleave': this.onPointerLeave,
            'edge:pointermove': this.onPointerMove,
            'combo:pointerenter': this.onPointerEnter,
            'combo:pointerleave': this.onPointerLeave,
            'combo:pointermove': this.onPointerMove,
            afterremoveitem: this.onPointerLeave,
            contextmenu: this.onPointerLeave,
            'node:drag': this.onPointerLeave,
        };
    }
    init(graph) {
        super.init(graph);
        const className = this.options.className;
        const tooltip = (0, dom_util_1.createDom)(`<div class='${className || 'g6-component-tooltip'}'></div>`);
        (0, dom_util_1.modifyCSS)(tooltip, {
            position: 'absolute',
            visibility: 'hidden',
            display: 'none',
        });
        if (this.options.trigger !== 'click') {
            tooltip.addEventListener('pointerenter', (e) => {
                (0, dom_util_1.modifyCSS)(tooltip, {
                    visibility: 'visible',
                    display: 'unset',
                });
            });
            tooltip.addEventListener('pointerleave', (e) => {
                this.hideTooltip();
            });
        }
        //`container` string type in v5
        let container = this.options.container;
        if (!container) {
            container = this.graph.container;
        }
        if ((0, util_1.isString)(container)) {
            container = document.getElementById(container);
        }
        container.appendChild(tooltip);
        this.tooltip = tooltip;
        this.container = container;
    }
    onClick(e) {
        if (e.itemId &&
            e.itemType &&
            this.options.itemTypes.indexOf(e.itemType) === -1)
            return;
        const { itemId } = e;
        // click the same item twice, tooltip will be hidden
        if (this.currentTarget === itemId) {
            this.currentTarget = null;
            this.hideTooltip();
            this.graph.emit('tooltipchange', { itemId: itemId, action: 'hide' });
        }
        else {
            this.currentTarget = itemId;
            this.showTooltip(e);
            // this.graph.emit('tooltipchange', { itemId: itemId, action: 'show' });
        }
    }
    onPointerEnter(e) {
        if (e.itemId &&
            e.itemType &&
            this.options.itemTypes.indexOf(e.itemType) === -1)
            return;
        const { itemId } = e;
        this.currentTarget = itemId;
        this.showTooltip(e);
        this.graph.emit('tooltipchange', { itemId: e.itemId, action: 'show' });
    }
    onPointerMove(e) {
        if (e.itemId &&
            e.itemType &&
            this.options.itemTypes.indexOf(e.itemType) === -1)
            return;
        if (!this.currentTarget || e.itemId === this.currentTarget) {
            return;
        }
        this.showTooltip(e);
    }
    onPointerLeave() {
        this.hideTooltip();
        this.graph.emit('tooltipchange', {
            itemId: this.currentTarget,
            action: 'hide',
        });
        this.currentTarget = null;
    }
    clearContainer() {
        this.container.innerHTML = '';
    }
    showTooltip(e) {
        return __awaiter(this, void 0, void 0, function* () {
            clearTimeout(this.hidenTimer);
            if (!e.itemId) {
                return;
            }
            if (e.itemType && this.options.itemTypes.indexOf(e.itemType) === -1)
                return;
            const tooltip = this.options.getContent(e);
            const tooltipDom = this.tooltip;
            // modify the position first because of the async function
            (0, dom_util_1.modifyCSS)(tooltipDom, {
                display: 'unset',
            });
            const res = this.updatePosition(e);
            (0, dom_util_1.modifyCSS)(tooltipDom, {
                visibility: 'visible',
                left: `${res.x}px`,
                top: `${res.y}px`,
            });
            if ((0, util_1.isString)(tooltip)) {
                tooltipDom.innerHTML = tooltip;
            }
            else if (tooltip instanceof HTMLDivElement) {
                this.clearContainer();
                this.container.appendChild(tooltip);
            }
            else {
                //promise type
                this.asyncTooltip = null; //avoid trigger many time
                if ((0, util_1.isString)(this.options.loadingContent)) {
                    tooltipDom.innerHTML = this.options.loadingContent;
                }
                else {
                    tooltipDom.innerHTML = this.options.loadingContent.outerHTML;
                }
                if (!this.asyncTooltip || e.itemId !== this.currentAsyncTarget) {
                    this.currentAsyncTarget = e.itemId;
                    this.asyncTooltip = yield this.options.getContent(e);
                }
                if (e.itemId != this.currentAsyncTarget) {
                    return;
                }
                if ((0, util_1.isString)(this.asyncTooltip)) {
                    tooltipDom.innerHTML = this.asyncTooltip;
                }
                else {
                    this.clearContainer();
                    this.container.appendChild(this.asyncTooltip);
                }
            }
        });
    }
    hideTooltip() {
        const tooltip = this.tooltip;
        if (tooltip) {
            this.hidenTimer = setTimeout(() => {
                (0, dom_util_1.modifyCSS)(tooltip, { visibility: 'hidden', display: 'none' });
            }, 100);
            tooltip.addEventListener('pointerenter', (e) => {
                clearTimeout(this.hidenTimer);
            });
            tooltip.addEventListener('pointerleave', (e) => {
                (0, dom_util_1.modifyCSS)(tooltip, { visibility: 'hidden', display: 'none' });
            });
        }
    }
    updatePosition(e) {
        const shouldBegin = this.options.shouldBegin;
        const tooltip = this.tooltip;
        if (!shouldBegin(e)) {
            (0, dom_util_1.modifyCSS)(tooltip, {
                visibility: 'hidden',
                display: 'none',
            });
            return;
        }
        const graph = this.graph;
        const width = graph.getSize()[0];
        const height = graph.getSize()[1];
        const offsetX = this.options.offsetX || 0;
        const offsetY = this.options.offsetY || 0;
        let point = {
            x: e.viewport.x,
            y: e.viewport.y,
        };
        const fixToNode = this.options.fixToNode;
        //handle `fixToNode` config
        if (e.itemType && e.itemType === 'node' && fixToNode) {
            const itemBBox = graph.getRenderBBox(e.itemId);
            const itemWidth = itemBBox.max[0] - itemBBox.min[0];
            const itemHeight = itemBBox.max[1] - itemBBox.min[1];
            if ((0, util_1.isString)(fixToNode)) {
                switch (fixToNode) {
                    case 'right': {
                        point = {
                            x: itemBBox.min[0] + itemWidth * 1,
                            y: itemBBox.min[1] - itemHeight * 0.5,
                        };
                        break;
                    }
                    case 'rightTop':
                    case 'topRight': {
                        point = {
                            x: itemBBox.min[0] + itemWidth * 1,
                            y: itemBBox.min[1] - itemHeight * 1,
                        };
                        break;
                    }
                    case 'rightBottom':
                    case 'bottomRight': {
                        point = {
                            x: itemBBox.min[0] + itemWidth * 1,
                            y: itemBBox.min[1],
                        };
                        break;
                    }
                    case 'bottom': {
                        point = {
                            x: itemBBox.min[0] + itemWidth * 0,
                            y: itemBBox.min[1],
                        };
                        break;
                    }
                    case 'top': {
                        const tooltipBBox = tooltip.getBoundingClientRect();
                        point = {
                            x: itemBBox.min[0] + itemWidth * 0,
                            y: itemBBox.min[1] - tooltipBBox.height,
                        };
                        break;
                    }
                    case 'left': {
                        const tooltipBBox = tooltip.getBoundingClientRect();
                        point = {
                            x: itemBBox.min[0] - tooltipBBox.width,
                            y: itemBBox.min[1] - itemHeight * 0.5,
                        };
                        break;
                    }
                    case 'leftTop':
                    case 'topLeft': {
                        const tooltipBBox = tooltip.getBoundingClientRect();
                        point = {
                            x: itemBBox.min[0] - tooltipBBox.width,
                            y: itemBBox.min[1] - itemHeight * 1,
                        };
                        break;
                    }
                    case 'leftBottom':
                    case 'bottomLeft': {
                        const tooltipBBox = tooltip.getBoundingClientRect();
                        point = {
                            x: itemBBox.min[0] - tooltipBBox.width,
                            y: itemBBox.min[1],
                        };
                        break;
                    }
                    default:
                        //right
                        point = {
                            x: itemBBox.min[0] + itemWidth * 1,
                            y: itemBBox.min[1] - itemHeight * 0.5,
                        };
                        console.warn(`The '${this.options.fixToNode}' fixToNode position configuration is not supported, please use 'top'|'left'| 'right'| 'bottom'| 'topLeft'| 'leftTop'| 'topRight'| 'rightTop'| 'bottomLeft'| 'leftBottom'| 'bottomRight'| 'rightBottom', or use array to config, like: [0,5,1]`);
                        break;
                }
            }
            else if ((0, util_1.isArray)(fixToNode) && fixToNode.length >= 2) {
                point = {
                    x: itemBBox.min[0] + itemWidth * fixToNode[0],
                    y: itemBBox.min[1] - itemHeight * (1 - fixToNode[1]),
                };
            }
        }
        const { x, y } = this.graph.getViewportByCanvas(point);
        const graphContainer = this.graph.container;
        const res = {
            x: x + graphContainer.offsetLeft + offsetX,
            y: y + graphContainer.offsetTop + offsetY,
        };
        //tooltip dom bbox
        const bbox = tooltip.getBoundingClientRect();
        if (x + bbox.width + offsetX > width) {
            res.x -= bbox.width + offsetX;
        }
        if (y + bbox.height + offsetY > height) {
            res.y -= bbox.height + offsetY;
            if (res.y < 0) {
                res.y = 0;
            }
        }
        return res;
    }
    destroy() {
        this.container.removeChild(this.tooltip);
    }
}
exports.Tooltip = Tooltip;
//# sourceMappingURL=index.js.map