"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mixColor = exports.SubjectThemeSolver = void 0;
const util_1 = require("@antv/util");
const color_1 = __importDefault(require("color"));
const colors_1 = require("@ant-design/colors");
const shape_1 = require("../../util/shape");
const base_1 = require("./base");
class SubjectThemeSolver extends base_1.BaseThemeSolver {
    solver(options, themes) {
        const { base = 'light', baseColor = '#fff', specification } = options;
        const baseSpec = themes[base];
        const mergedSpec = Object.assign({}, baseSpec);
        if (specification) {
            ['node', 'edge', 'combo'].forEach((itemType) => {
                if (!specification[itemType])
                    return;
                const { palette, dataTypeField } = specification[itemType];
                // merge the custom part spec and the built-in spec
                const { styles: [baseStyles], } = baseSpec[itemType];
                let mergedStyles;
                if ((0, util_1.isArray)(palette)) {
                    mergedStyles = palette.map((subjectColor) => {
                        const colorSet = getColorsWithSubjectColor(subjectColor, baseColor, base);
                        return this.getMergeStylesFromColorSet(itemType, colorSet, baseStyles);
                    });
                }
                else {
                    mergedStyles = { others: baseStyles };
                    Object.keys(palette).forEach((dataType) => {
                        const subjectColor = palette[dataType];
                        const colorSet = getColorsWithSubjectColor(subjectColor, baseColor, base);
                        mergedStyles[dataType] = this.getMergeStylesFromColorSet(itemType, colorSet, baseStyles);
                    });
                }
                mergedSpec[itemType] = {
                    dataTypeField,
                    palette,
                    styles: mergedStyles,
                };
            });
            mergedSpec['canvas'] = Object.assign(Object.assign({}, baseSpec.canvas), specification.canvas);
        }
        mergedSpec['canvas'] = Object.assign(Object.assign({}, baseSpec.canvas), specification.canvas);
        return mergedSpec;
    }
    getMergeStylesFromColorSet(itemType, colorSet, baseStyles) {
        const mergedStatesStyles = Object.assign({}, baseStyles);
        if (itemType === 'node') {
            mergedStatesStyles.default = (0, shape_1.mergeStyles)([
                baseStyles.default,
                {
                    keyShape: {
                        stroke: colorSet.mainStroke,
                        fill: colorSet.mainFill,
                    },
                },
            ]);
            mergedStatesStyles.selected = (0, shape_1.mergeStyles)([
                baseStyles.selected,
                {
                    keyShape: {
                        fill: colorSet.selectedFill,
                        stroke: colorSet.selectedStroke,
                        shadowColor: colorSet.selectedStroke,
                    },
                },
            ]);
            mergedStatesStyles.active = (0, shape_1.mergeStyles)([
                baseStyles.active,
                {
                    keyShape: {
                        fill: colorSet.activeFill,
                        stroke: colorSet.activeStroke,
                        shadowColor: colorSet.mainStroke,
                    },
                },
            ]);
            mergedStatesStyles.highlight = (0, shape_1.mergeStyles)([
                baseStyles.highlight,
                {
                    keyShape: {
                        fill: colorSet.highlightFill,
                        stroke: colorSet.highlightStroke,
                    },
                },
            ]);
            mergedStatesStyles.inactive = (0, shape_1.mergeStyles)([
                baseStyles.inactive,
                {
                    keyShape: {
                        fill: colorSet.inactiveFill,
                        stroke: colorSet.inactiveStroke,
                    },
                },
            ]);
            mergedStatesStyles.disable = (0, shape_1.mergeStyles)([
                baseStyles.disable,
                {
                    keyShape: {
                        fill: colorSet.disableFill,
                        stroke: colorSet.disableStroke,
                    },
                },
            ]);
        }
        else if (itemType === 'edge') {
            mergedStatesStyles.default = (0, shape_1.mergeStyles)([
                baseStyles.default,
                {
                    keyShape: {
                        stroke: colorSet.edgeMainStroke,
                    },
                },
            ]);
            mergedStatesStyles.selected = (0, shape_1.mergeStyles)([
                baseStyles.selected,
                {
                    keyShape: {
                        stroke: colorSet.edgeSelectedStroke,
                        shadowColor: colorSet.edgeSelectedStroke,
                    },
                },
            ]);
            mergedStatesStyles.active = (0, shape_1.mergeStyles)([
                baseStyles.active,
                {
                    keyShape: {
                        stroke: colorSet.edgeActiveStroke,
                    },
                },
            ]);
            mergedStatesStyles.highlight = (0, shape_1.mergeStyles)([
                baseStyles.highlight,
                {
                    keyShape: {
                        stroke: colorSet.edgeHighlightStroke,
                    },
                },
            ]);
            mergedStatesStyles.inactive = (0, shape_1.mergeStyles)([
                baseStyles.inactive,
                {
                    keyShape: {
                        stroke: colorSet.edgeInactiveStroke,
                    },
                },
            ]);
            mergedStatesStyles.disable = (0, shape_1.mergeStyles)([
                baseStyles.disable,
                {
                    keyShape: {
                        stroke: colorSet.edgeDisableStroke,
                    },
                },
            ]);
        }
        else if (itemType === 'combo') {
            mergedStatesStyles.default = (0, shape_1.mergeStyles)([
                baseStyles.default,
                {
                    keyShape: {
                        fill: colorSet.comboMainFill,
                        stroke: colorSet.comboMainStroke,
                    },
                },
            ]);
            mergedStatesStyles.selected = (0, shape_1.mergeStyles)([
                baseStyles.selected,
                {
                    keyShape: {
                        stroke: colorSet.comboSelectedStroke,
                        fill: colorSet.comboSelectedFill,
                        shadowColor: colorSet.comboSelectedStroke,
                    },
                },
            ]);
            mergedStatesStyles.active = (0, shape_1.mergeStyles)([
                baseStyles.active,
                {
                    keyShape: {
                        stroke: colorSet.comboActiveStroke,
                        fill: colorSet.comboActiveFill,
                    },
                },
            ]);
            mergedStatesStyles.highlight = (0, shape_1.mergeStyles)([
                baseStyles.highlight,
                {
                    keyShape: {
                        stroke: colorSet.comboHighlightStroke,
                        fill: colorSet.comboHighlightFill,
                    },
                },
            ]);
            mergedStatesStyles.inactive = (0, shape_1.mergeStyles)([
                baseStyles.inactive,
                {
                    keyShape: {
                        stroke: colorSet.comboInactiveStroke,
                        fill: colorSet.comboInactiveFill,
                    },
                },
            ]);
            mergedStatesStyles.disable = (0, shape_1.mergeStyles)([
                baseStyles.disable,
                {
                    keyShape: {
                        stroke: colorSet.comboDisableStroke,
                        fill: colorSet.comboDisableFill,
                    },
                },
            ]);
        }
        return mergedStatesStyles;
    }
}
exports.SubjectThemeSolver = SubjectThemeSolver;
/**
 * get the mix color of backColor and frontColor with alpah
 * @param backColor background color
 * @param frontColor foreground color
 * @param frontAlpha the opacity of foreground color
 */
const mixColor = (backColor, frontColor, frontAlpha) => {
    const bc = (0, color_1.default)(backColor);
    const fc = (0, color_1.default)(frontColor);
    return (0, color_1.default)([
        (1 - frontAlpha) * bc.red() + frontAlpha * fc.red(),
        (1 - frontAlpha) * bc.green() + frontAlpha * fc.green(),
        (1 - frontAlpha) * bc.blue() + frontAlpha * fc.blue(),
    ]).rgb();
};
exports.mixColor = mixColor;
const getColorsWithDefaultTheme = (subjectColor, backColor = '#fff', disableColor = 'rgb(150, 150, 150)') => {
    const subjectColor005 = (0, exports.mixColor)(backColor, subjectColor, 0.05)
        .rgb()
        .toString();
    const subjectColor01 = (0, exports.mixColor)(backColor, subjectColor, 0.1)
        .rgb()
        .toString();
    const subjectColor02 = (0, exports.mixColor)(backColor, subjectColor, 0.2)
        .rgb()
        .toString();
    const subjectColor04 = (0, exports.mixColor)(backColor, subjectColor, 0.4)
        .rgb()
        .toString();
    const disableColor002 = (0, exports.mixColor)(backColor, disableColor, 0.02)
        .rgb()
        .toString();
    const disableColor005 = (0, exports.mixColor)(backColor, disableColor, 0.05)
        .rgb()
        .toString();
    const disableColor01 = (0, exports.mixColor)(backColor, disableColor, 0.1)
        .rgb()
        .toString();
    const disableColor02 = (0, exports.mixColor)(backColor, disableColor, 0.2)
        .rgb()
        .toString();
    const disableColor03 = (0, exports.mixColor)(backColor, disableColor, 0.3)
        .rgb()
        .toString();
    const paletteFromSubject = (0, colors_1.generate)(subjectColor, {
        theme: 'default',
        backgroundColor: backColor,
    });
    const subjectHex = (0, color_1.default)(subjectColor).hex().toLowerCase();
    const subjectIdx = paletteFromSubject.indexOf(subjectHex);
    let deeperSubject = subjectColor;
    if (subjectIdx !== -1) {
        deeperSubject = paletteFromSubject[subjectIdx + 1];
    }
    return {
        // for nodes
        mainStroke: subjectColor,
        mainFill: subjectColor01,
        activeStroke: subjectColor,
        activeFill: subjectColor005,
        inactiveStroke: subjectColor04,
        inactiveFill: subjectColor005,
        selectedStroke: subjectColor,
        selectedFill: subjectColor01,
        highlightStroke: deeperSubject,
        highlightFill: subjectColor02,
        disableStroke: disableColor03,
        disableFill: disableColor005,
        // for edges
        edgeMainStroke: subjectColor04,
        edgeActiveStroke: subjectColor,
        edgeInactiveStroke: disableColor02,
        edgeSelectedStroke: subjectColor,
        edgeHighlightStroke: subjectColor,
        edgeDisableStroke: disableColor01,
        // for combos
        comboMainStroke: disableColor03,
        comboMainFill: disableColor002,
        comboActiveStroke: subjectColor,
        comboActiveFill: subjectColor005,
        comboInactiveStroke: disableColor03,
        comboInactiveFill: disableColor002,
        comboSelectedStroke: subjectColor,
        comboSelectedFill: disableColor002,
        comboHighlightStroke: deeperSubject,
        comboHighlightFill: disableColor002,
        comboDisableStroke: disableColor02,
        comboDisableFill: disableColor005,
    };
};
const getColorsWithDarkTheme = (subjectColor, backColor = '#fff', disableColor = '#777') => {
    const subjectColor02 = (0, exports.mixColor)(backColor, subjectColor, 0.2)
        .rgb()
        .toString();
    const subjectColor03 = (0, exports.mixColor)(backColor, subjectColor, 0.3)
        .rgb()
        .toString();
    const subjectColor06 = (0, exports.mixColor)(backColor, subjectColor, 0.6)
        .rgb()
        .toString();
    const subjectColor08 = (0, exports.mixColor)(backColor, subjectColor, 0.8)
        .rgb()
        .toString();
    const disableColor02 = (0, exports.mixColor)(backColor, disableColor, 0.2)
        .rgb()
        .toString();
    const disableColor025 = (0, exports.mixColor)(backColor, disableColor, 0.25)
        .rgb()
        .toString();
    const disableColor03 = (0, exports.mixColor)(backColor, disableColor, 0.3)
        .rgb()
        .toString();
    const disableColor04 = (0, exports.mixColor)(backColor, disableColor, 0.4)
        .rgb()
        .toString();
    const disableColor05 = (0, exports.mixColor)(backColor, disableColor, 0.5)
        .rgb()
        .toString();
    const paletteFromSubject = (0, colors_1.generate)(subjectColor, {
        theme: 'dark',
        backgroundColor: backColor,
    });
    const subjectHex = (0, color_1.default)(subjectColor).hex().toLowerCase();
    const subjectIdx = paletteFromSubject.indexOf(subjectHex);
    let deeperSubject = subjectColor;
    if (subjectIdx !== -1) {
        deeperSubject = paletteFromSubject[subjectIdx + 1];
    }
    return {
        // for nodes
        mainStroke: subjectColor08,
        mainFill: subjectColor02,
        activeStroke: subjectColor,
        activeFill: subjectColor03,
        inactiveStroke: subjectColor08,
        inactiveFill: subjectColor02,
        selectedStroke: subjectColor,
        selectedFill: subjectColor02,
        highlightStroke: subjectColor,
        highlightFill: subjectColor06,
        disableStroke: disableColor05,
        disableFill: disableColor025,
        // for edges
        edgeMainStroke: subjectColor08,
        edgeActiveStroke: subjectColor,
        edgeInactiveStroke: disableColor,
        edgeSelectedStroke: subjectColor,
        edgeHighlightStroke: subjectColor,
        edgeDisableStroke: disableColor03,
        // for combos
        comboMainStroke: disableColor04,
        comboMainFill: disableColor025,
        comboActiveStroke: subjectColor,
        comboActiveFill: disableColor02,
        comboInactiveStroke: disableColor04,
        comboInactiveFill: disableColor025,
        comboSelectedStroke: subjectColor,
        comboSelectedFill: disableColor02,
        comboHighlightStroke: deeperSubject,
        comboHighlightFill: disableColor025,
        comboDisableStroke: disableColor04,
        comboDisableFill: disableColor02,
    };
};
/**
 * get the set of colors according to the subject color and background color
 * @param subjectColor the subject color
 * @param backColor background color
 * @param disableColor the color for disable state
 */
const getColorsWithSubjectColor = (subjectColor, backColor = '#fff', theme = 'light') => {
    if (theme !== 'light')
        return getColorsWithDarkTheme(subjectColor, backColor, '#777');
    return getColorsWithDefaultTheme(subjectColor, backColor, 'rgb(150, 150, 150)');
};
const getColorSetsBySubjectColors = (subjectColors, backColor = '#fff', theme = 'light') => {
    const sets = [];
    subjectColors.forEach((sColor) => {
        sets.push(getColorsWithSubjectColor(sColor, backColor, theme));
    });
    return sets;
};
//# sourceMappingURL=subject.js.map