import { IGraph } from './graph';
export interface IPluginBaseConfig {
    container?: HTMLDivElement | string | null;
    className?: string;
    graph?: IGraph;
    [key: string]: any;
}
export declare abstract class Plugin {
    key: string;
    private events;
    options: IPluginBaseConfig;
    destroyed: boolean;
    graph: IGraph;
    /**
     * Constructor for the Plugin Base Class.
     * @param options options for the plugin
     */
    constructor(options?: IPluginBaseConfig);
    /**
     * Update the options for the plugin.
     * @param options
     */
    updateCfgs(options?: IPluginBaseConfig): void;
    /**
     * Get the defualt options of the plugin.
     */
    getDefaultCfgs(): {};
    /**
     * Init.
     */
    init(graph: IGraph): void;
    /**
     * Returns the key(graph event name) value(listener) map.
     * Should be implemented by the plugin.
     */
    getEvents(): {};
    /**
     * Destroy method, should be implemented by the plugin.
     */
    destroy(): void;
}
export interface PluginRegistry {
    [key: string]: typeof Plugin;
}
