"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Plugin = void 0;
const util_1 = require("@antv/util");
class Plugin {
    /**
     * Constructor for the Plugin Base Class.
     * @param options options for the plugin
     */
    constructor(options) {
        this.options = (0, util_1.deepMix)(this.getDefaultCfgs(), options);
        this.events = {};
        this.destroyed = false;
    }
    /**
     * Update the options for the plugin.
     * @param options
     */
    updateCfgs(options) {
        this.options = (0, util_1.deepMix)(this.options, options);
        this.init(this.graph);
    }
    /**
     * Get the defualt options of the plugin.
     */
    getDefaultCfgs() {
        return {};
    }
    /**
     * Init.
     */
    init(graph) {
        this.graph = graph;
    }
    /**
     * Returns the key(graph event name) value(listener) map.
     * Should be implemented by the plugin.
     */
    getEvents() {
        return {};
    }
    /**
     * Destroy method, should be implemented by the plugin.
     */
    destroy() {
        const { graph } = this;
        const events = this.events;
        (0, util_1.each)(events, (v, k) => {
            graph.off(k, v);
        });
        this.events = null;
        this.options = null;
        this.destroyed = true;
    }
}
exports.Plugin = Plugin;
//# sourceMappingURL=plugin.js.map