import { DisplayObject, Group, IAnimation } from '@antv/g';
import { AnimateTiming, IAnimates } from '../types/animate';
import { ItemShapeStyles } from '../types/item';
/**
 * Initial(timing = show) shape animation start from init shape styles, and end to the shape's style config.
 */
export declare const getShapeAnimateBeginStyles: (shape: any) => {
    fontSize?: undefined;
    opacity?: undefined;
    strokeOpacity?: undefined;
    offsetDistance?: undefined;
} | {
    opacity: number;
    strokeOpacity: number;
    lineWidth: number;
    offsetDistance: number;
    lineDash: any[];
    fontSize?: undefined;
} | {
    opacity: number;
    strokeOpacity: number;
    lineWidth: number;
    offsetDistance: number;
    r: number;
    fontSize?: undefined;
} | {
    opacity: number;
    strokeOpacity: number;
    lineWidth: number;
    offsetDistance: number;
    rx: number;
    ry: number;
    fontSize?: undefined;
} | {
    fontSize: number;
    opacity: number;
    strokeOpacity: number;
    offsetDistance: number;
} | {
    opacity: number;
    strokeOpacity: number;
    lineWidth: number;
    offsetDistance: number;
    width: number;
    height: number;
    fontSize?: undefined;
};
/**
 * Initial(timing = show) group animation start from GROUP_ANIMATE_STYLES[0], and end to GROUP_ANIMATE_STYLES[1].
 */
export declare const GROUP_ANIMATE_STYLES: {
    opacity: number;
    transform: string;
}[];
/**
 * Default animate options for different timing.
 */
export declare const DEFAULT_ANIMATE_CFG: {
    buildIn: {
        duration: number;
        easing: string;
        iterations: number;
        delay: number;
        fill: string;
    };
    show: {
        duration: number;
        easing: string;
        iterations: number;
        fill: string;
    };
    update: {
        duration: number;
        easing: string;
        iterations: number;
        fill: string;
    };
    zoom: {
        duration: number;
        easing: string;
        iterations: number;
        delay: number;
        fill: string;
    };
};
/**
 * Handle shape and group animations.
 * Should be called after canvas ready and shape appended.
 * @param animates
 * @param mergedStyles
 * @param shapeMap
 * @param group
 * @param timing timing to match 'when' in the animate config in style
 * @returns
 */
export declare const animateShapes: (animates: IAnimates, mergedStyles: ItemShapeStyles, shapeMap: {
    [shapeId: string]: DisplayObject<any, any>;
}, group: Group, timing?: AnimateTiming, changedStates?: string[], onAnimatesFrame?: Function, onAnimatesEnd?: Function) => IAnimation[];
export declare const getAnimatesExcludePosition: (animates: any) => any;
export declare const fadeIn: (id: any, shape: any, style: any, hiddenShape: any, animateConfig: any) => void;
export declare const fadeOut: (id: any, shape: any, hiddenShapeMap: any, animateConfig: any) => void;
/**
 * Make the animation to the end frame and clear it from the target shape.
 * @param animation
 */
export declare const stopAnimate: (animation: IAnimation) => Promise<any>;
