"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.changeRenderer = exports.createCanvas = void 0;
const g_1 = require("@antv/g");
const g_canvas_1 = require("@antv/g-canvas");
const g_svg_1 = require("@antv/g-svg");
const g_webgl_1 = require("@antv/g-webgl");
const g_plugin_3d_1 = require("@antv/g-plugin-3d");
const g_plugin_dragndrop_1 = require("@antv/g-plugin-dragndrop");
/**
 * Create a canvas
 * @param {RendererName} rendererType
 * @param {string | HTMLElement} container
 * @param {number} width
 * @param {number} height
 * @param {number} pixelRatio optional
 * @returns
 */
const createCanvas = (rendererType, container, width, height, pixelRatio, background) => {
    let renderer;
    switch (rendererType.toLowerCase()) {
        case 'svg':
            renderer = new g_svg_1.Renderer();
            break;
        case 'webgl-3d':
        case 'webgl':
            renderer = new g_webgl_1.Renderer();
            renderer.registerPlugin(new g_plugin_3d_1.Plugin());
            break;
        default:
            renderer = new g_canvas_1.Renderer();
            break;
    }
    renderer.registerPlugin(new g_plugin_dragndrop_1.Plugin({
        isDocumentDraggable: true,
        isDocumentDroppable: true,
        dragstartDistanceThreshold: 10,
        dragstartTimeThreshold: 100,
    }));
    return new g_1.Canvas({
        container,
        width,
        height,
        devicePixelRatio: pixelRatio,
        renderer,
        supportsMutipleCanvasesInOneContainer: true,
        background,
    });
};
exports.createCanvas = createCanvas;
/**
 * Change renderer type for the canvas.
 * @param rendererType renderer type
 * @param canvas Canvas instance
 * @returns
 */
const changeRenderer = (rendererType, canvas) => {
    let renderer;
    switch (rendererType.toLowerCase()) {
        case 'svg':
            renderer = new g_svg_1.Renderer();
            break;
        case 'webgl-3d':
        case 'webgl':
            renderer = new g_webgl_1.Renderer();
            renderer.registerPlugin(new g_plugin_3d_1.Plugin());
            break;
        default:
            renderer = new g_canvas_1.Renderer();
            break;
    }
    renderer.registerPlugin(new g_plugin_dragndrop_1.Plugin());
    canvas.setRenderer(renderer);
    return canvas;
};
exports.changeRenderer = changeRenderer;
//# sourceMappingURL=canvas.js.map