import { NodeUserModel } from 'types';
import { ID, TreeData } from '@antv/graphlib';
import { NodeUserModelData } from 'types/node';
import { GraphCore, GraphData } from '../types/data';
import { IGraph } from '../types/graph';
/**
 * Deconstruct data and distinguish nodes and combos from graphcore data.
 * @param data data from graphcore
 * @returns
 */
export declare const deconstructData: (data: any) => {
    nodes: any[];
    edges: any;
    combos: any[];
};
/**
 * Depth first search begin from nodes in graphCore data.
 * @param graphCore graphlib data structure
 * @param nodes begin nodes
 * @param fn will be called while visiting each node
 * @param mode 'TB' - visit from top to bottom; 'BT' - visit from bottom to top;
 * @returns
 */
export declare const graphCoreTreeDfs: (graphCore: GraphCore, nodes: NodeUserModel[], fn: any, mode?: 'TB' | 'BT', treeKey?: string, stopFns?: {
    stopBranchFn?: (node: NodeUserModel) => boolean;
    stopAllFn?: (node: NodeUserModel) => boolean;
}) => void;
/**
 * Depth first search begin from nodes in g6 graph data.
 * @param graph G6 graph instance
 * @param nodes begin nodes
 * @param fn will be called while visiting each node
 * @param mode 'TB' - visit from top to bottom; 'BT' - visit from bottom to top;
 * @returns
 */
export declare const graphComboTreeDfs: (graph: IGraph, nodes: NodeUserModel[], fn: any, mode?: 'TB' | 'BT') => void;
/**
 * Depth first search begin from nodes in g6 graph data, and then search the ancestirs from begin nodes.
 * @param graph G6 graph instance
 * @param graphCore graphlib data structure
 * @param nodes begin ndoes
 * @param fn will be called while visiting each node
 * @param mode 'TB' - visit from top to bottom; 'BT' - visit from bottom to top;
 * @returns
 */
export declare const traverseAncestorsAndSucceeds: (graph: IGraph, graphCore: GraphCore, nodes: NodeUserModel[], fn: any, mode?: 'TB' | 'BT') => void;
export declare const traverseGraphAncestors: (graph: IGraph, nodes: NodeUserModel[], fn: any) => void;
/**
 * Traverse the ancestors from the begin nodes.
 * @param graphCore graphlib data structure
 * @param nodes begin ndoes
 * @param fn will be called while visiting each node
 */
export declare const traverseAncestors: (graphCore: any, nodes: any, fn: any) => void;
/**
 * Whether the test succeed is a parent of the test parent.
 * @param graph G6 graph instance
 * @param testParent id of the node to be the parent
 * @param parentId id of the node to be the succeed
 * @returns
 */
export declare const isSucceed: (graph: any, testParent: any, testSucceed: any) => boolean;
/**
 * Check the validation if move the child into the subtree of parent. e.g. If the new parent (toBeAncestorId) is a succeed of new child (toBeSucceedId), it is invalid.
 * @param graph G6 graph instance
 * @param toBeSucceedId id of the new child to check if it is valid to be a child of the node with id toBeAncestorId
 * @param toBeAncestorId id of the new parent to check if it is valid to be a parent of the node with id toBeSucceedId
 * @returns
 */
export declare const validateComboStrucutre: (graph: any, toBeSucceedId: any, toBeAncestorId: any) => boolean;
/**
 * Transform tree graph data into graph data. Edges from parent-child structure.
 * @param treeData Tree structured data or an array of it.
 * @returns Graph formatted data object with nodes, edges and combos.
 */
export declare const treeData2GraphData: (treeData: TreeData<NodeUserModelData> | TreeData<NodeUserModelData>[]) => {
    nodes: any[];
    edges: any[];
    combos: any[];
};
/**
 * Transform graph data into tree graph data.
 * @param nodeMap This function will cache the nodes into nodeMap
 * @param graphData Graph data.
 * @param propRootIds Ids of root nodes. There should be at least one node for each connected component, or the first node in a connected component will be added to the roots array.
 * @returns
 */
export declare const graphData2TreeData: (nodeMap: {
    [id: string]: any;
}, graphData: GraphData, propRootIds?: ID[]) => any[];
/**
 * Travere a tree data from top to bottom.
 * @param treeData
 * @param callback
 */
export declare const traverse: (treeData: any, callback: any) => void;
