import { BehaviorRegistry } from '../types/behavior';
import { ThemeRegistry } from '../types/theme';
import { NodeRegistry } from '../types/node';
import { EdgeRegistry } from '../types/edge';
import { LayoutRegistry } from '../types/layout';
import { PluginRegistry } from '../types/plugin';
import Graph from '../runtime/graph';
/**
 * Extend graph class with custom libs (extendLibrary), and extendLibrary will be merged into useLib.
 * B1 is the Behavior lib from user, B2 is the Behavior lib of the graph to be extended(built-in graph)
 * TODO: more templates, and might be merged to be two templates for the whole extendLibrary
 * @param GraphClass graph class to be extended
 * @param extendLibrary custom libs to extend
 * @returns extended graph class
 */
export declare const extend: <B1 extends BehaviorRegistry, B2 extends BehaviorRegistry, T1 extends ThemeRegistry, T2 extends ThemeRegistry>(GraphClass: {
    new (spec: import("..").Specification<B2, T2>): Graph<B2, T2>;
}, extendLibrary: {
    behaviors?: B1;
    themeSolvers?: T1;
    nodes?: NodeRegistry;
    edges?: EdgeRegistry;
    layouts?: LayoutRegistry;
    plugins?: PluginRegistry;
}) => {
    new (spec: import("..").Specification<B1 & B2, T1 & T2>): Graph<B1 & B2, T1 & T2>;
};
