"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateShapes3D = exports.upsertShape3D = exports.createShape3D = void 0;
const g_plugin_3d_1 = require("@antv/g-plugin-3d");
const shape_1 = require("./shape");
const GeometryTagMap = {
    sphere: g_plugin_3d_1.SphereGeometry,
    cubic: g_plugin_3d_1.CubeGeometry,
    plane: g_plugin_3d_1.PlaneGeometry,
};
const GEOMETRY_SIZE = 10;
const createShape3D = (type, style, id, device) => {
    // It is not a 3d Shape but 2d.
    if (!GeometryTagMap[type]) {
        return (0, shape_1.createShape)(type, style, id);
    }
    // materialType: 'lambert' | 'phong' | 'basic', TODO: type
    const _a = style, { materialType = 'lambert' } = _a, otherStyles = __rest(_a, ["materialType"]);
    if (!device.GeometryCache) {
        device.GeometryCache = {};
    }
    // Share the same geometry & material between meshes.
    let cachedGeometry = device.GeometryCache[type];
    if (!cachedGeometry) {
        cachedGeometry = device.GeometryCache[type] = new GeometryTagMap[type](device, {
            radius: GEOMETRY_SIZE,
            latitudeBands: 32,
            longitudeBands: 32,
        });
    }
    if (!device.MaterialCache) {
        device.MaterialCache = {};
    }
    if (!device.MaterialCache[materialType]) {
        switch (materialType) {
            case 'basic':
                device.MaterialCache[materialType] = new g_plugin_3d_1.MeshBasicMaterial(device);
                break;
            case 'phong': {
                const materialProps = {
                    shininess: 30,
                };
                device.MaterialCache[materialType] = new g_plugin_3d_1.MeshPhongMaterial(device, materialProps);
            }
            case 'lambert':
            default: {
                device.MaterialCache[materialType] = new g_plugin_3d_1.MeshLambertMaterial(device);
                break;
            }
        }
    }
    const shape = new g_plugin_3d_1.Mesh({
        style: Object.assign(Object.assign({ visibility: 'visible', strokeOpacity: 0 }, otherStyles), { geometry: cachedGeometry, material: device.MaterialCache[materialType] }),
        id,
    });
    // set origin for shape
    shape.setOrigin(0, 0, 0);
    // Scale the shape to the correct size.
    switch (type) {
        case 'cube':
            shape.scale([
                style.width / GEOMETRY_SIZE,
                style.height / GEOMETRY_SIZE,
                style.depth / GEOMETRY_SIZE,
            ]);
            break;
        case 'plane':
            shape.scale([style.width / GEOMETRY_SIZE, style.depth / GEOMETRY_SIZE]);
            break;
        case 'sphere':
        default: {
            const scaling = (style.r || style.radius) / GEOMETRY_SIZE;
            shape.scale(scaling);
        }
    }
    return shape;
};
exports.createShape3D = createShape3D;
const upsertShape3D = (type, id, style, shapeMap, device) => {
    let shape = shapeMap[id];
    if (!shape) {
        // @ts-ignore
        shape = (0, exports.createShape3D)(type, style, id, device);
        shape.setAttribute(shape_1.LOCAL_BOUNDS_DIRTY_FLAG_KEY, true);
    }
    else if (shape.nodeName !== type) {
        shape.remove();
        // @ts-ignore
        shape = (0, exports.createShape3D)(type, style, id, device);
        shape.setAttribute(shape_1.LOCAL_BOUNDS_DIRTY_FLAG_KEY, true);
    }
    else {
        const updateStyles = {};
        const oldStyles = shape.attributes;
        Object.keys(style).forEach((key) => {
            if (oldStyles[key] !== style[key]) {
                updateStyles[key] = style[key];
                shape.style[key] = style[key];
            }
        });
        if ((0, shape_1.isStyleAffectBBox)(type, updateStyles)) {
            shape.setAttribute(shape_1.LOCAL_BOUNDS_DIRTY_FLAG_KEY, true);
        }
    }
    shapeMap[id] = shape;
    return shape;
};
exports.upsertShape3D = upsertShape3D;
/**
 * Update shapes in the intersaction of prevShapeMap and newShapeMap;
 * Remove shapes in the prevShapeMap - newShapeMap (if removeDiff is true);
 * Add shapes in the newShapeMap - prevShapeMap;
 * @param prevShapeMap previous shape map
 * @param newShapeMap new shape map
 * @param group container group
 * @returns merged shape map
 */
const updateShapes3D = (prevShapeMap, newShapeMap, group, removeDiff = true, shouldUpdate = () => true) => {
    const tolalMap = Object.assign(Object.assign({}, prevShapeMap), newShapeMap);
    const finalShapeMap = Object.assign({}, prevShapeMap);
    Object.keys(tolalMap).forEach((id) => {
        const prevShape = prevShapeMap[id];
        const newShape = newShapeMap[id];
        if (newShape && !shouldUpdate(id))
            return;
        if (prevShape && newShape) {
            // update intersaction
            finalShapeMap[id] = newShape;
            if (prevShape !== newShape) {
                prevShape.remove();
            }
            group.appendChild(newShape);
        }
        else if (!prevShape && newShape) {
            // add newShapeMap - prevShapeMap
            finalShapeMap[id] = newShape;
            group.appendChild(newShape);
        }
        else if (prevShape && !newShape && removeDiff) {
            // remove prevShapeMap - newShapeMap
            delete finalShapeMap[id];
            prevShape.remove();
        }
    });
    return finalShapeMap;
};
exports.updateShapes3D = updateShapes3D;
//# sourceMappingURL=shape3d.js.map