"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getZoomLevel = exports.formatLodStrategy = void 0;
/**
 * Format lodStrategy to the pattern that ratio 1 (primary level) at level 0, and higher the ratio, higher the level.
 * @param lodStrategy
 * @returns
 */
const formatLodStrategy = (lodStrategy) => {
    if (!lodStrategy)
        return;
    const { levels, animateCfg } = lodStrategy;
    if (!levels)
        return;
    const primaryLevel = levels.find((level) => level.primary);
    if (!primaryLevel)
        return;
    const primaryIndex = levels.indexOf(primaryLevel);
    const formattedLevels = {};
    levels.forEach((level, i) => {
        formattedLevels[i - primaryIndex] = level.zoomRange;
    });
    return {
        animateCfg,
        levels: formattedLevels,
    };
};
exports.formatLodStrategy = formatLodStrategy;
/**
 * Get zoom level idx in levels array.
 * @param levels
 * @param zoom
 * @returns
 */
const getZoomLevel = (levels, zoom) => {
    let level = 0;
    Object.keys(levels).forEach((idx) => {
        const zoomRange = levels[idx];
        if (zoom >= zoomRange[0] && zoom < zoomRange[1])
            level = Number(idx);
    });
    return level;
};
exports.getZoomLevel = getZoomLevel;
//# sourceMappingURL=zoom.js.map